@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,36 +9,30 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "quickitemnodeinstance.h"
-
 #include "qt5nodeinstanceserver.h"
 
+#include <qmlprivategate.h>
+
+#include <QQmlProperty>
 #include <QQmlExpression>
 #include <QQuickView>
 #include <cmath>
 
-#include <private/qquicktextinput_p.h>
-#include <private/qquicktextedit_p.h>
-
 #include <QHash>
 
 #include <QDebug>
@@ -191,20 +185,6 @@ bool QuickItemNodeInstance::hasContent() const
     return childItemsHaveContent(quickItem());
 }
 
-static void disableTextCursor(QQuickItem *item)
-{
-    foreach (QQuickItem *childItem, item->childItems())
-        disableTextCursor(childItem);
-
-    QQuickTextInput *textInput = qobject_cast<QQuickTextInput*>(item);
-    if (textInput)
-        textInput->setCursorVisible(false);
-
-    QQuickTextEdit *textEdit = qobject_cast<QQuickTextEdit*>(item);
-    if (textEdit)
-        textEdit->setCursorVisible(false);
-}
-
 void QuickItemNodeInstance::doComponentComplete()
 {
     ObjectNodeInstance::doComponentComplete();
@@ -213,7 +193,7 @@ void QuickItemNodeInstance::doComponentComplete()
     if (contentItemProperty.isValid())
         m_contentItem = contentItemProperty.read().value<QQuickItem*>();
 
-    disableTextCursor(quickItem());
+    QmlPrivateGate::disableTextCursor(quickItem());
 
     DesignerSupport::emitComponentCompleteSignalForAttachedProperty(quickItem());
 
@@ -464,6 +444,7 @@ void QuickItemNodeInstance::updateDirtyNodesRecursive(QQuickItem *parentItem) co
             updateDirtyNodesRecursive(childItem);
     }
 
+    QmlPrivateGate::disableNativeTextRendering(parentItem);
     DesignerSupport::updateDirtyNode(parentItem);
 }
 
@@ -599,14 +580,17 @@ void QuickItemNodeInstance::reparent(const ObjectNodeInstance::Pointer &oldParen
             setPropertyVariant("y", y());
     }
 
-    refresh();
-    DesignerSupport::updateDirtyNode(quickItem());
+    if (quickItem()->parentItem()) {
+        refresh();
+        if (quickItem()->window())
+            DesignerSupport::updateDirtyNode(quickItem());
 
-    if (instanceIsValidLayoutable(oldParentInstance, oldParentProperty))
-        oldParentInstance->refreshLayoutable();
+        if (instanceIsValidLayoutable(oldParentInstance, oldParentProperty))
+            oldParentInstance->refreshLayoutable();
 
-    if (instanceIsValidLayoutable(newParentInstance, newParentProperty))
-        newParentInstance->refreshLayoutable();
+        if (instanceIsValidLayoutable(newParentInstance, newParentProperty))
+            newParentInstance->refreshLayoutable();
+    }
 }
 
 void QuickItemNodeInstance::setPropertyVariant(const PropertyName &name, const QVariant &value)
@@ -662,6 +646,16 @@ void QuickItemNodeInstance::setPropertyBinding(const PropertyName &name, const Q
 
     refresh();
 
+    /* Evaluate properties of the root item in the context of the dummy context if they contain parent.
+     * This is done manually because we cannot "overwrite" the parent property
+     */
+
+    if (isRootNodeInstance() && expression.contains(QLatin1String("parent."))) {
+        QQmlExpression qmlContextExpression(context(), nodeInstanceServer()->dummyContextObject(), expression);
+        QVariant value = qmlContextExpression.evaluate();
+        setPropertyVariant(name, value);
+    }
+
     if (isInLayoutable())
         parentInstance()->refreshLayoutable();
 }
@@ -698,7 +692,7 @@ void QuickItemNodeInstance::resetProperty(const PropertyName &name)
     if (name == "y")
         m_y = 0.0;
 
-    DesignerSupport::resetAnchor(quickItem(), name);
+    DesignerSupport::resetAnchor(quickItem(), QString::fromUtf8(name));
 
     if (name == "anchors.fill") {
         resetHorizontal();
@@ -735,30 +729,31 @@ bool QuickItemNodeInstance::isAnchoredByChildren() const
 
 bool QuickItemNodeInstance::hasAnchor(const PropertyName &name) const
 {
-    return DesignerSupport::hasAnchor(quickItem(), name);
+    return DesignerSupport::hasAnchor(quickItem(), QString::fromUtf8(name));
 }
 
 static bool isValidAnchorName(const PropertyName &name)
 {
-    static PropertyNameList anchorNameList(PropertyNameList() << "anchors.top"
-                                                    << "anchors.left"
-                                                    << "anchors.right"
-                                                    << "anchors.bottom"
-                                                    << "anchors.verticalCenter"
-                                                    << "anchors.horizontalCenter"
-                                                    << "anchors.fill"
-                                                    << "anchors.centerIn"
-                                                    << "anchors.baseline");
+    static PropertyNameList anchorNameList({"anchors.top",
+                                            "anchors.left",
+                                            "anchors.right",
+                                            "anchors.bottom",
+                                            "anchors.verticalCenter",
+                                            "anchors.horizontalCenter",
+                                            "anchors.fill",
+                                            "anchors.centerIn",
+                                            "anchors.baseline"});
 
     return anchorNameList.contains(name);
 }
 
 QPair<PropertyName, ServerNodeInstance> QuickItemNodeInstance::anchor(const PropertyName &name) const
 {
-    if (!isValidAnchorName(name) || !DesignerSupport::hasAnchor(quickItem(), name))
+    if (!isValidAnchorName(name) || !DesignerSupport::hasAnchor(quickItem(), QString::fromUtf8(name)))
         return ObjectNodeInstance::anchor(name);
 
-    QPair<QString, QObject*> nameObjectPair = DesignerSupport::anchorLineTarget(quickItem(), name, context());
+    QPair<QString, QObject*> nameObjectPair =
+            DesignerSupport::anchorLineTarget(quickItem(), QString::fromUtf8(name), context());
 
     QObject *targetObject = nameObjectPair.second;
     PropertyName targetName = nameObjectPair.first.toUtf8();
