@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -66,7 +61,6 @@ struct FancyMainWindowPrivate
     QAction m_menuSeparator1;
     QAction m_menuSeparator2;
     QAction m_resetLayoutAction;
-    QDockWidget *m_toolBarDockWidget;
     QAction m_autoHideTitleBars;
 };
 
@@ -159,13 +153,16 @@ public:
 
         m_floatButton = new DockWidgetTitleButton(this);
         m_floatButton->setIcon(q->style()->standardIcon(QStyle::SP_TitleBarNormalButton, &opt, q));
-        m_floatButton->setAccessibleName(QDockWidget::tr("Float"));
-        m_floatButton->setAccessibleDescription(QDockWidget::tr("Undocks and re-attaches the dock widget"));
 
         m_closeButton = new DockWidgetTitleButton(this);
         m_closeButton->setIcon(q->style()->standardIcon(QStyle::SP_TitleBarCloseButton, &opt, q));
+
+#ifndef QT_NO_ACCESSIBILITY
+        m_floatButton->setAccessibleName(QDockWidget::tr("Float"));
+        m_floatButton->setAccessibleDescription(QDockWidget::tr("Undocks and re-attaches the dock widget"));
         m_closeButton->setAccessibleName(QDockWidget::tr("Close"));
         m_closeButton->setAccessibleDescription(QDockWidget::tr("Closes the dock widget"));
+#endif
 
         setActive(false);
 
@@ -341,7 +338,6 @@ FancyMainWindowPrivate::FancyMainWindowPrivate(FancyMainWindow *parent) :
     m_menuSeparator1(0),
     m_menuSeparator2(0),
     m_resetLayoutAction(FancyMainWindow::tr("Reset to Default Layout"), 0),
-    m_toolBarDockWidget(0),
     m_autoHideTitleBars(FancyMainWindow::tr("Automatically Hide View Title Bars"), 0)
 {
     m_menuSeparator1.setSeparator(true);
@@ -557,14 +553,4 @@ void FancyMainWindow::setDockActionsVisible(bool v)
     d->m_resetLayoutAction.setVisible(v);
 }
 
-QDockWidget *FancyMainWindow::toolBarDockWidget() const
-{
-    return d->m_toolBarDockWidget;
-}
-
-void FancyMainWindow::setToolBarDockWidget(QDockWidget *dock)
-{
-    d->m_toolBarDockWidget = dock;
-}
-
 } // namespace Utils
