@@ -52,6 +52,8 @@ public:
     CMakeBuildConfiguration(ProjectExplorer::Target *parent);
     ~CMakeBuildConfiguration();
 
+    void cmakeFilesChanged();
+
     bool isEnabled() const override;
     QString disabledReason() const override;
 
@@ -68,6 +70,7 @@ public:
     CMakeConfig cmakeConfiguration() const;
 
     QString error() const;
+    QString warning() const;
 
     BuildDirManager *buildDirManager() const;
 
@@ -83,6 +86,7 @@ public:
 
 signals:
     void errorOccured(const QString &message);
+    void warningOccured(const QString &message);
 
     void parsingStarted();
     void dataAvailable();
@@ -97,16 +101,18 @@ private:
     void setCurrentCMakeConfiguration(const QList<ConfigModel::DataItem> &items);
 
     void setError(const QString &message);
+    void setWarning(const QString &message);
 
     CMakeConfig m_configuration;
     QString m_error;
+    QString m_warning;
 
     mutable QList<CMakeConfigItem> m_completeConfigurationCache;
 
     BuildDirManager *m_buildDirManager = nullptr;
 
     friend class CMakeBuildSettingsWidget;
-    friend class CMakeProject;
+    friend class CMakeProjectManager::CMakeProject;
 };
 
 class CMakeBuildConfigurationFactory : public ProjectExplorer::IBuildConfigurationFactory
