@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -88,14 +83,8 @@ public:
 
     bool findPageById(const Id id, int *pageIndex) const
     {
-        for (int j = 0; j < pages.size(); ++j) {
-            IOptionsPage *page = pages.at(j);
-            if (page->id() == id) {
-                *pageIndex = j;
-                return true;
-            }
-        }
-        return false;
+        *pageIndex = Utils::indexOf(pages, Utils::equal(&IOptionsPage::id, id));
+        return *pageIndex != -1;
     }
 
     Id id;
@@ -114,8 +103,8 @@ public:
     CategoryModel(QObject *parent = 0);
     ~CategoryModel();
 
-    int rowCount(const QModelIndex &parent = QModelIndex()) const;
-    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
 
     void setPages(const QList<IOptionsPage*> &pages,
                   const QList<IOptionsPageProvider *> &providers);
@@ -262,7 +251,7 @@ public:
     {}
 
 protected:
-    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;
+    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override;
 };
 
 bool CategoryFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
@@ -348,6 +337,7 @@ public:
     {
         setFrameStyle(QFrame::NoFrame | QFrame::Plain);
         viewport()->setAutoFillBackground(false);
+        setWidgetResizable(true);
     }
 private:
     void resizeEvent(QResizeEvent *event) override
@@ -396,7 +386,6 @@ private:
             return 0;
         return list.first()->sizeHint().width();
     }
-
 };
 
 
@@ -465,7 +454,6 @@ void SettingsDialog::showPage(const Id pageId)
         for (int i = 0; i < categories.size(); ++i) {
             Category *category = categories.at(i);
             if (category->providers.isEmpty()) {  // no providers
-                ensureCategoryWidget(category);
                 if (category->findPageById(initialPageId, &initialPageIndex)) {
                     initialCategoryIndex = i;
                     break;
