@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,37 +9,33 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef MAINWINDOW_H
-#define MAINWINDOW_H
+#pragma once
 
 #include "icontext.h"
 #include "icore.h"
-#include "dialogs/newdialog.h"
 
 #include <utils/appmainwindow.h>
+#include <utils/dropsupport.h>
 
 #include <QMap>
 #include <QColor>
 
+#include <functional>
+
 QT_BEGIN_NAMESPACE
 class QSettings;
 class QPrinter;
@@ -48,7 +44,6 @@ QT_END_NAMESPACE
 
 namespace Core {
 
-class ActionManager;
 class StatusBarWidget;
 class EditorManager;
 class ExternalToolManager;
@@ -56,19 +51,17 @@ class DocumentManager;
 class HelpManager;
 class IDocument;
 class IWizardFactory;
+class JsExpander;
 class MessageManager;
-class MimeDatabase;
 class ModeManager;
 class ProgressManager;
 class NavigationWidget;
 class RightPaneWidget;
 class SettingsDatabase;
-class VariableManager;
 class VcsManager;
 
 namespace Internal {
 
-class ActionManagerPrivate;
 class FancyTabWidget;
 class GeneralSettings;
 class ProgressManagerPrivate;
@@ -79,6 +72,7 @@ class StatusBarManager;
 class VersionDialog;
 class WindowSupport;
 class SystemEditor;
+class SystemSettings;
 
 class MainWindow : public Utils::AppMainWindow
 {
@@ -93,54 +87,50 @@ public:
     void aboutToShutdown();
 
     IContext *contextObject(QWidget *widget);
-    void addContextObject(IContext *contex);
-    void removeContextObject(IContext *contex);
+    void addContextObject(IContext *context);
+    void removeContextObject(IContext *context);
 
-    Core::IDocument *openFiles(const QStringList &fileNames, ICore::OpenFilesFlags flags);
+    IDocument *openFiles(const QStringList &fileNames,
+                         ICore::OpenFilesFlags flags,
+                         const QString &workingDirectory = QString());
 
     inline SettingsDatabase *settingsDatabase() const { return m_settingsDatabase; }
     virtual QPrinter *printer() const;
     IContext * currentContextObject() const;
     QStatusBar *statusBar() const;
 
-    void updateAdditionalContexts(const Context &remove, const Context &add);
+    void updateAdditionalContexts(const Context &remove, const Context &add,
+                                  ICore::ContextPriority priority);
 
     void setSuppressNavigationWidget(bool suppress);
 
     void setOverrideColor(const QColor &color);
 
-    bool isNewItemDialogRunning() const;
+    QStringList additionalAboutInformation() const;
+    void appendAboutInformation(const QString &line);
+
+    void addPreCloseListener(const std::function<bool()> &listener);
 
 signals:
-    void windowActivated();
     void newItemDialogRunningChanged();
 
 public slots:
-    void newFile();
     void openFileWith();
     void exit();
 
-    void showNewItemDialog(const QString &title,
-                           const QList<IWizardFactory *> &factories,
-                           const QString &defaultLocation = QString(),
-                           const QVariantMap &extraVariables = QVariantMap());
-
-    bool showOptionsDialog(Id category = Id(), Id page = Id(), QWidget *parent = 0);
+    bool showOptionsDialog(Id page = Id(), QWidget *parent = 0);
 
     bool showWarningWithOptions(const QString &title, const QString &text,
                                 const QString &details = QString(),
-                                Id settingsCategory = Id(),
                                 Id settingsId = Id(),
                                 QWidget *parent = 0);
 
 protected:
-    virtual void changeEvent(QEvent *e);
     virtual void closeEvent(QCloseEvent *event);
 
-private slots:
+private:
     void openFile();
     void aboutToShowRecentFiles();
-    void openRecentFile();
     void setFocusToEditor();
     void saveAll();
     void aboutQtCreator();
@@ -148,11 +138,10 @@ private slots:
     void updateFocusWidget(QWidget *old, QWidget *now);
     void setSidebarVisible(bool visible);
     void destroyVersionDialog();
-    void openDroppedFiles(const QStringList &files);
+    void openDroppedFiles(const QList<Utils::DropSupport::FileSpec> &files);
     void restoreWindowState();
     void newItemDialogFinished();
 
-private:
     void updateContextObject(const QList<IContext *> &context);
     void updateContext();
 
@@ -163,33 +152,33 @@ private:
     void writeSettings();
 
     ICore *m_coreImpl;
-    Context m_additionalContexts;
+    QStringList m_aboutInformation;
+    Context m_highPrioAdditionalContexts;
+    Context m_lowPrioAdditionalContexts;
     SettingsDatabase *m_settingsDatabase;
     mutable QPrinter *m_printer;
     WindowSupport *m_windowSupport;
-    ActionManager *m_actionManager;
     EditorManager *m_editorManager;
     ExternalToolManager *m_externalToolManager;
     MessageManager *m_messageManager;
     ProgressManagerPrivate *m_progressManager;
-    VariableManager *m_variableManager;
+    JsExpander *m_jsExpander;
     VcsManager *m_vcsManager;
     StatusBarManager *m_statusBarManager;
     ModeManager *m_modeManager;
-    MimeDatabase *m_mimeDatabase;
     HelpManager *m_helpManager;
     FancyTabWidget *m_modeStack;
     NavigationWidget *m_navigationWidget;
     RightPaneWidget *m_rightPaneWidget;
-    Core::StatusBarWidget *m_outputView;
+    StatusBarWidget *m_outputView;
     VersionDialog *m_versionDialog;
-    QPointer<NewDialog> m_newDialog;
 
     QList<IContext *> m_activeContext;
 
     QMap<QWidget *, IContext *> m_contextWidgets;
 
     GeneralSettings *m_generalSettings;
+    SystemSettings *m_systemSettings;
     ShortcutSettings *m_shortcutSettings;
     ToolSettings *m_toolSettings;
     MimeTypeSettings *m_mimeTypeSettings;
@@ -205,12 +194,12 @@ private:
     QAction *m_optionsAction;
     QAction *m_toggleSideBarAction;
     QAction *m_toggleModeSelectorAction;
+    QAction *m_themeAction;
 
     QToolButton *m_toggleSideBarButton;
     QColor m_overrideColor;
+    QList<std::function<bool()>> m_preCloseListeners;
 };
 
 } // namespace Internal
 } // namespace Core
-
-#endif // MAINWINDOW_H
