@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,28 +9,25 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "variablechooser.h"
 #include "coreconstants.h"
 
+#include <coreplugin/coreicons.h>
+
 #include <utils/fancylineedit.h> // IconButton
 #include <utils/headerviewstretcher.h> // IconButton
 #include <utils/macroexpander.h>
@@ -72,11 +69,13 @@ public:
         setAttribute(Qt::WA_MacShowFocusRect, false);
         setIndentation(indentation() * 7/10);
         header()->hide();
-        new Utils::HeaderViewStretcher(header(), 0);
+        new HeaderViewStretcher(header(), 0);
     }
 
     void contextMenuEvent(QContextMenuEvent *ev);
 
+    void currentChanged(const QModelIndex &current, const QModelIndex &previous);
+
 private:
     VariableChooserPrivate *m_target;
 };
@@ -89,9 +88,9 @@ public:
 
     void createIconButton()
     {
-        m_iconButton = new Utils::IconButton;
-        m_iconButton->setPixmap(QPixmap(QLatin1String(":/core/images/replace.png")));
-        m_iconButton->setToolTip(tr("Insert variable"));
+        m_iconButton = new IconButton;
+        m_iconButton->setPixmap(Core::Icons::REPLACE.pixmap());
+        m_iconButton->setToolTip(VariableChooser::tr("Insert Variable"));
         m_iconButton->hide();
         connect(m_iconButton.data(), static_cast<void(QAbstractButton::*)(bool)>(&QAbstractButton::clicked),
                 this, &VariableChooserPrivate::updatePositionAndShow);
@@ -115,7 +114,7 @@ public:
     QPointer<QLineEdit> m_lineEdit;
     QPointer<QTextEdit> m_textEdit;
     QPointer<QPlainTextEdit> m_plainTextEdit;
-    QPointer<Utils::IconButton> m_iconButton;
+    QPointer<IconButton> m_iconButton;
 
     VariableTreeView *m_variableTree;
     QLabel *m_variableDescription;
@@ -127,9 +126,8 @@ class VariableGroupItem : public TreeItem
 {
 public:
     VariableGroupItem()
-        : m_chooser(0)
+        : m_chooser(0), m_populated(false)
     {
-        setLazy(true);
     }
 
     QVariant data(int column, int role) const
@@ -143,16 +141,23 @@ public:
         return QVariant();
     }
 
-    void populate()
+    bool canFetchMore() const
+    {
+        return !m_populated;
+    }
+
+    void fetchMore()
     {
         if (MacroExpander *expander = m_provider())
             populateGroup(expander);
+        m_populated = true;
     }
 
     void populateGroup(MacroExpander *expander);
 
 public:
     VariableChooserPrivate *m_chooser; // Not owned.
+    bool m_populated;
     MacroExpanderProvider m_provider;
 };
 
@@ -168,8 +173,16 @@ public:
                 return m_variable;
         }
 
-        if (role == Qt::ToolTipRole)
-            return m_expander->variableDescription(m_variable.toUtf8());
+        if (role == Qt::ToolTipRole) {
+            const QByteArray var = m_variable.toUtf8();
+            QString description = m_expander->variableDescription(var);
+            QString value;
+            if (!m_expander->isPrefixVariable(var))
+                value = m_expander->value(var).toHtmlEscaped();
+            if (!value.isEmpty())
+                description += QLatin1String("<p>") + VariableChooser::tr("Current Value: %1").arg(value);
+            return description;
+        }
 
         if (role == UnexpandedTextRole)
             return QString(QLatin1String("%{") + m_variable + QLatin1Char('}'));
@@ -197,17 +210,17 @@ void VariableTreeView::contextMenuEvent(QContextMenuEvent *ev)
     QAction *insertExpandedAction = 0;
 
     if (unexpandedText.isEmpty()) {
-        insertUnexpandedAction = menu.addAction(tr("Insert unexpanded value"));
+        insertUnexpandedAction = menu.addAction(VariableChooser::tr("Insert Unexpanded Value"));
         insertUnexpandedAction->setEnabled(false);
     } else {
-        insertUnexpandedAction = menu.addAction(tr("Insert \"%1\"").arg(unexpandedText));
+        insertUnexpandedAction = menu.addAction(VariableChooser::tr("Insert \"%1\"").arg(unexpandedText));
     }
 
     if (expandedText.isEmpty()) {
-        insertExpandedAction = menu.addAction(tr("Insert expanded value"));
+        insertExpandedAction = menu.addAction(VariableChooser::tr("Insert Expanded Value"));
         insertExpandedAction->setEnabled(false);
     } else {
-        insertExpandedAction = menu.addAction(tr("Insert \"%1\"").arg(expandedText));
+        insertExpandedAction = menu.addAction(VariableChooser::tr("Insert \"%1\"").arg(expandedText));
     }
 
 
@@ -219,31 +232,39 @@ void VariableTreeView::contextMenuEvent(QContextMenuEvent *ev)
         m_target->insertText(expandedText);
 }
 
+void VariableTreeView::currentChanged(const QModelIndex &current, const QModelIndex &previous)
+{
+    m_target->updateDescription(current);
+    QTreeView::currentChanged(current, previous);
+}
+
 VariableChooserPrivate::VariableChooserPrivate(VariableChooser *parent)
     : q(parent),
       m_lineEdit(0),
       m_textEdit(0),
-      m_plainTextEdit(0)
+      m_plainTextEdit(0),
+      m_iconButton(0),
+      m_variableTree(0),
+      m_variableDescription(0)
 {
     m_defaultDescription = VariableChooser::tr("Select a variable to insert.");
 
     m_variableTree = new VariableTreeView(q, this);
-    m_variableTree->setModel(&m_model);
-
     m_variableDescription = new QLabel(q);
+
+    m_variableTree->setModel(&m_model);
     m_variableDescription->setText(m_defaultDescription);
     m_variableDescription->setMinimumSize(QSize(0, 60));
     m_variableDescription->setAlignment(Qt::AlignLeft|Qt::AlignTop);
     m_variableDescription->setWordWrap(true);
     m_variableDescription->setAttribute(Qt::WA_MacSmallSize);
+    m_variableDescription->setTextInteractionFlags(Qt::TextBrowserInteraction);
 
     QVBoxLayout *verticalLayout = new QVBoxLayout(q);
     verticalLayout->setContentsMargins(3, 3, 3, 12);
     verticalLayout->addWidget(m_variableTree);
     verticalLayout->addWidget(m_variableDescription);
 
-    connect(m_variableTree, &QTreeView::clicked,
-            this, &VariableChooserPrivate::updateDescription);
     connect(m_variableTree, &QTreeView::activated,
             this, &VariableChooserPrivate::handleItemActivated);
     connect(qobject_cast<QApplication *>(qApp), &QApplication::focusChanged,
@@ -387,7 +408,8 @@ void VariableChooser::addSupportForChildWidgets(QWidget *parent, MacroExpander *
  */
 void VariableChooserPrivate::updateDescription(const QModelIndex &index)
 {
-    m_variableDescription->setText(m_model.data(index, Qt::ToolTipRole).toString());
+    if (m_variableDescription)
+        m_variableDescription->setText(m_model.data(index, Qt::ToolTipRole).toString());
 }
 
 /*!
@@ -534,7 +556,7 @@ static bool handleEscapePressed(QKeyEvent *ke, QWidget *widget)
 {
     if (ke->key() == Qt::Key_Escape && !ke->modifiers()) {
         ke->accept();
-        QTimer::singleShot(0, widget, SLOT(close()));
+        QTimer::singleShot(0, widget, &QWidget::close);
         return true;
     }
     return false;
