@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -423,12 +418,8 @@ Scope *CheckSymbols::enclosingScope() const
                 return funDef->symbol;
 
         } else if (TemplateDeclarationAST *templateDeclaration = ast->asTemplateDeclaration()) {
-            if (DeclarationAST *decl = templateDeclaration->declaration) {
-                if (FunctionDefinitionAST *funDef = decl->asFunctionDefinition()) {
-                    if (funDef->symbol)
-                        return funDef->symbol;
-                }
-            }
+            if (templateDeclaration->symbol)
+                return templateDeclaration->symbol;
 
         } else if (CompoundStatementAST *blockStmt = ast->asCompoundStatement()) {
             if (blockStmt->symbol)
@@ -526,15 +517,16 @@ bool CheckSymbols::visit(SimpleDeclarationAST *ast)
                     if (funTy->isVirtual()
                             || (nameAST->asDestructorName()
                                 && hasVirtualDestructor(_context.lookupType(funTy->enclosingScope())))) {
-                        addUse(nameAST, SemanticHighlighter::VirtualMethodUse);
+                        addUse(nameAST, SemanticHighlighter::VirtualFunctionDeclarationUse);
                         declrIdNameAST = nameAST;
                     } else if (maybeAddFunction(_context.lookup(decl->name(),
                                                                 decl->enclosingScope()),
-                                                nameAST, funTy->argumentCount())) {
+                                                nameAST, funTy->argumentCount(),
+                                                FunctionDeclaration)) {
                         declrIdNameAST = nameAST;
 
                         // Add a diagnostic message if non-virtual function has override/final marker
-                        if ((_usages.back().kind != SemanticHighlighter::VirtualMethodUse)) {
+                        if ((_usages.back().kind != SemanticHighlighter::VirtualFunctionDeclarationUse)) {
                             if (funTy->isOverride())
                                 warning(declrIdNameAST, QCoreApplication::translate(
                                             "CPlusplus::CheckSymbols", "Only virtual functions can be marked 'override'"));
@@ -627,7 +619,7 @@ bool CheckSymbols::visit(CallAST *ast)
                         accept(tId->template_argument_list);
                     }
 
-                    if (!maybeAddFunction(candidates, memberName, argumentCount)
+                    if (!maybeAddFunction(candidates, memberName, argumentCount, FunctionCall)
                             && highlightCtorDtorAsType) {
                         expr = ast->base_expression;
                     }
@@ -650,7 +642,7 @@ bool CheckSymbols::visit(CallAST *ast)
                         typeOfExpression(textOf(idExpr), enclosingScope(),
                                          TypeOfExpression::Preprocess);
 
-                    if (!maybeAddFunction(candidates, exprName, argumentCount)
+                    if (!maybeAddFunction(candidates, exprName, argumentCount, FunctionCall)
                             && highlightCtorDtorAsType) {
                         expr = ast->base_expression;
                     }
@@ -713,7 +705,8 @@ bool CheckSymbols::visit(NewExpressionAST *ast)
                 }
             }
 
-            maybeAddFunction(_context.lookup(nameAST->name, scope), nameAST, arguments);
+            maybeAddFunction(_context.lookup(nameAST->name, scope), nameAST, arguments,
+                             FunctionCall);
         }
     }
 
@@ -809,7 +802,7 @@ void CheckSymbols::checkName(NameAST *ast, Scope *scope)
 
             if (klass) {
                 if (hasVirtualDestructor(_context.lookupType(klass))) {
-                    addUse(ast, SemanticHighlighter::VirtualMethodUse);
+                    addUse(ast, SemanticHighlighter::VirtualFunctionDeclarationUse);
                 } else {
                     bool added = false;
                     if (highlightCtorDtorAsType && maybeType(ast->name))
@@ -866,7 +859,7 @@ bool CheckSymbols::visit(QualifiedNameAST *ast)
         if (binding && ast->unqualified_name) {
             if (ast->unqualified_name->asDestructorName() != 0) {
                 if (hasVirtualDestructor(binding)) {
-                    addUse(ast->unqualified_name, SemanticHighlighter::VirtualMethodUse);
+                    addUse(ast->unqualified_name, SemanticHighlighter::VirtualFunctionDeclarationUse);
                 } else {
                     bool added = false;
                     if (highlightCtorDtorAsType && maybeType(ast->name))
@@ -983,7 +976,8 @@ bool CheckSymbols::visit(MemInitializerAST *ast)
                                 for (ExpressionListAST *it = expr_list; it; it = it->next)
                                     ++arguments;
                             }
-                            maybeAddFunction(_context.lookup(nameAST->name, klass), nameAST, arguments);
+                            maybeAddFunction(_context.lookup(nameAST->name, klass),
+                                             nameAST, arguments, FunctionCall);
                         }
 
                         break;
@@ -1066,10 +1060,9 @@ bool CheckSymbols::visit(FunctionDefinitionAST *ast)
             if (fun->isVirtual()
                     || (declId->asDestructorName()
                         && hasVirtualDestructor(_context.lookupType(fun->enclosingScope())))) {
-                addUse(declId, SemanticHighlighter::VirtualMethodUse);
-            } else if (!maybeAddFunction(_context.lookup(fun->name(),
-                                                         fun->enclosingScope()),
-                                         declId, fun->argumentCount())) {
+                addUse(declId, SemanticHighlighter::VirtualFunctionDeclarationUse);
+            } else if (!maybeAddFunction(_context.lookup(fun->name(), fun->enclosingScope()),
+                                         declId, fun->argumentCount(), FunctionDeclaration)) {
                 processEntireDeclr = true;
             }
         }
@@ -1183,8 +1176,9 @@ bool CheckSymbols::isTemplateClass(Symbol *symbol) const
     if (symbol) {
         if (Template *templ = symbol->asTemplate()) {
             if (Symbol *declaration = templ->declaration()) {
-                if (declaration->isClass() || declaration->isForwardClassDeclaration())
-                    return true;
+                return declaration->isClass()
+                        || declaration->isForwardClassDeclaration()
+                        || declaration->isTypedef();
             }
         }
     }
@@ -1267,7 +1261,8 @@ bool CheckSymbols::maybeAddField(const QList<LookupItem> &candidates, NameAST *a
     return false;
 }
 
-bool CheckSymbols::maybeAddFunction(const QList<LookupItem> &candidates, NameAST *ast, unsigned argumentCount)
+bool CheckSymbols::maybeAddFunction(const QList<LookupItem> &candidates, NameAST *ast,
+                                    unsigned argumentCount, FunctionKind functionKind)
 {
     unsigned startToken = ast->firstToken();
     bool isDestructor = false;
@@ -1283,7 +1278,9 @@ bool CheckSymbols::maybeAddFunction(const QList<LookupItem> &candidates, NameAST
         return false;
 
     enum { Match_None, Match_TooManyArgs, Match_TooFewArgs, Match_Ok } matchType = Match_None;
-    Kind kind = SemanticHighlighter::FunctionUse;
+
+    Kind kind = functionKind == FunctionDeclaration ? SemanticHighlighter::FunctionDeclarationUse
+                                                    : SemanticHighlighter::FunctionUse;
     foreach (const LookupItem &r, candidates) {
         Symbol *c = r.declaration();
 
@@ -1305,27 +1302,34 @@ bool CheckSymbols::maybeAddFunction(const QList<LookupItem> &candidates, NameAST
                 if ((c = t->declaration()))
                     funTy = c->type()->asFunctionType();
         }
-        if (!funTy)
+        if (!funTy || funTy->isAmbiguous())
             continue; // TODO: add diagnostic messages and color call-operators calls too?
 
+        const bool isVirtual = funTy->isVirtual();
+        Kind matchingKind;
+        if (functionKind == FunctionDeclaration) {
+            matchingKind = isVirtual ? SemanticHighlighter::VirtualFunctionDeclarationUse
+                                     : SemanticHighlighter::FunctionDeclarationUse;
+        } else {
+            matchingKind = isVirtual ? SemanticHighlighter::VirtualMethodUse
+                                     : SemanticHighlighter::FunctionUse;
+        }
         if (argumentCount < funTy->minimumArgumentCount()) {
             if (matchType != Match_Ok) {
-                kind = funTy->isVirtual() ? SemanticHighlighter::VirtualMethodUse : SemanticHighlighter::FunctionUse;
+                kind = matchingKind;
                 matchType = Match_TooFewArgs;
             }
         } else if (argumentCount > funTy->argumentCount() && !funTy->isVariadic()) {
             if (matchType != Match_Ok) {
                 matchType = Match_TooManyArgs;
-                kind = funTy->isVirtual() ? SemanticHighlighter::VirtualMethodUse : SemanticHighlighter::FunctionUse;
+                kind = matchingKind;
             }
-        } else if (!funTy->isVirtual()) {
-            matchType = Match_Ok;
-            kind = SemanticHighlighter::FunctionUse;
-            //continue, to check if there is a matching candidate which is virtual
         } else {
             matchType = Match_Ok;
-            kind = SemanticHighlighter::VirtualMethodUse;
-            break;
+            kind = matchingKind;
+            if (isVirtual)
+                break;
+            // else continue, to check if there is a matching candidate which is virtual
         }
     }
 
@@ -1334,7 +1338,7 @@ bool CheckSymbols::maybeAddFunction(const QList<LookupItem> &candidates, NameAST
         if (highlightCtorDtorAsType
                 && (isConstructor || isDestructor)
                 && maybeType(ast->name)
-                && kind == SemanticHighlighter::FunctionUse) {
+                && kind == SemanticHighlighter::FunctionDeclarationUse) {
             return false;
         }
 
