@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -64,7 +59,7 @@ namespace Git {
 namespace Internal {
 
 GitEditorWidget::GitEditorWidget() :
-    m_changeNumberPattern(QLatin1String(CHANGE_PATTERN))
+    m_changeNumberPattern(CHANGE_PATTERN)
 {
     QTC_ASSERT(m_changeNumberPattern.isValid(), return);
     /* Diff format:
@@ -73,8 +68,8 @@ GitEditorWidget::GitEditorWidget() :
         --- a/src/plugins/git/giteditor.cpp
         +++ b/src/plugins/git/giteditor.cpp
     */
-    setDiffFilePattern(QRegExp(QLatin1String("^(?:diff --git a/|index |[+-]{3} (?:/dev/null|[ab]/(.+$)))")));
-    setLogEntryPattern(QRegExp(QLatin1String("^commit ([0-9a-f]{8})[0-9a-f]{32}")));
+    setDiffFilePattern(QRegExp("^(?:diff --git a/|index |[+-]{3} (?:/dev/null|[ab]/(.+$)))"));
+    setLogEntryPattern(QRegExp("^commit ([0-9a-f]{8})[0-9a-f]{32}"));
     setAnnotateRevisionTextFormat(tr("&Blame %1"));
     setAnnotatePreviousRevisionTextFormat(tr("Blame &Parent Revision %1"));
 }
@@ -86,11 +81,11 @@ QSet<QString> GitEditorWidget::annotationChanges() const
     if (txt.isEmpty())
         return changes;
     // Hunt for first change number in annotation: "<change>:"
-    QRegExp r(QLatin1String("^(" CHANGE_PATTERN ") "));
+    QRegExp r("^(" CHANGE_PATTERN ") ");
     QTC_ASSERT(r.isValid(), return changes);
     if (r.indexIn(txt) != -1) {
         changes.insert(r.cap(1));
-        r.setPattern(QLatin1String("\n(" CHANGE_PATTERN ") "));
+        r.setPattern("\n(" CHANGE_PATTERN ") ");
         QTC_ASSERT(r.isValid(), return changes);
         int pos = 0;
         while ((pos = r.indexIn(txt, pos)) != -1) {
@@ -124,71 +119,68 @@ BaseAnnotationHighlighter *GitEditorWidget::createAnnotationHighlighter(const QS
 8ca887aa (author               YYYY-MM-DD HH:MM:SS <offset>  <line>)<content>
 \endcode */
 
-static QString removeAnnotationDate(const QString &b)
+static QString sanitizeBlameOutput(const QString &b)
 {
     if (b.isEmpty())
         return b;
 
-    const QChar space(QLatin1Char(' '));
-    const int parenPos = b.indexOf(QLatin1Char(')'));
+    const bool omitDate = GitPlugin::client()->settings().boolValue(
+                GitSettings::omitAnnotationDateKey);
+    const QChar space(' ');
+    const int parenPos = b.indexOf(')');
     if (parenPos == -1)
         return b;
-    int datePos = parenPos;
 
     int i = parenPos;
     while (i >= 0 && b.at(i) != space)
         --i;
     while (i >= 0 && b.at(i) == space)
         --i;
-    int spaceCount = 0;
-    // i is now on timezone. Go back 3 spaces: That is where the date starts.
-    while (i >= 0) {
-        if (b.at(i) == space)
-            ++spaceCount;
-        if (spaceCount == 3) {
-            datePos = i;
-            break;
+    int stripPos = i + 1;
+    if (omitDate) {
+        int spaceCount = 0;
+        // i is now on timezone. Go back 3 spaces: That is where the date starts.
+        while (i >= 0) {
+            if (b.at(i) == space)
+                ++spaceCount;
+            if (spaceCount == 3) {
+                stripPos = i;
+                break;
+            }
+            --i;
         }
-        --i;
     }
-    if (datePos == 0)
-        return b;
 
     // Copy over the parts that have not changed into a new byte array
     QString result;
-    QTC_ASSERT(b.size() >= parenPos, return result);
     int prevPos = 0;
-    int pos = b.indexOf(QLatin1Char('\n'), 0) + 1;
+    int pos = b.indexOf('\n', 0) + 1;
     forever {
         QTC_CHECK(prevPos < pos);
         int afterParen = prevPos + parenPos;
-        result.append(b.mid(prevPos, datePos));
+        result.append(b.mid(prevPos, stripPos));
         result.append(b.mid(afterParen, pos - afterParen));
         prevPos = pos;
         QTC_CHECK(prevPos != 0);
         if (pos == b.size())
             break;
 
-        pos = b.indexOf(QLatin1Char('\n'), pos) + 1;
+        pos = b.indexOf('\n', pos) + 1;
         if (pos == 0) // indexOf returned -1
             pos = b.size();
     }
     return result;
 }
 
-void GitEditorWidget::setPlainTextFiltered(const QString &text)
+void GitEditorWidget::setPlainText(const QString &text)
 {
     QString modText = text;
-    GitPlugin *plugin = GitPlugin::instance();
     // If desired, filter out the date from annotation
     switch (contentType())
     {
-    case AnnotateOutput: {
-        const bool omitAnnotationDate = plugin->settings().boolValue(GitSettings::omitAnnotationDateKey);
-        if (omitAnnotationDate)
-            modText = removeAnnotationDate(text);
+    case AnnotateOutput:
+        modText = sanitizeBlameOutput(text);
         break;
-    }
     default:
         break;
     }
@@ -196,56 +188,30 @@ void GitEditorWidget::setPlainTextFiltered(const QString &text)
     textDocument()->setPlainText(modText);
 }
 
-void GitEditorWidget::commandFinishedGotoLine(bool ok, int exitCode, const QVariant &v)
-{
-    reportCommandFinished(ok, exitCode, v);
-    if (ok && v.type() == QVariant::Int) {
-        const int line = v.toInt();
-        if (line >= 0)
-            gotoLine(line);
-    }
-}
-
 void GitEditorWidget::checkoutChange()
 {
-    GitPlugin::instance()->gitClient()->stashAndCheckout(
-                sourceWorkingDirectory(), m_currentChange);
+    GitPlugin::client()->stashAndCheckout(sourceWorkingDirectory(), m_currentChange);
 }
 
 void GitEditorWidget::resetChange(const QByteArray &resetType)
 {
-    const QString workingDir = sourceWorkingDirectory();
-
-    GitClient *client = GitPlugin::instance()->gitClient();
-    if (resetType == "hard"
-            && client->gitStatus(workingDir, StatusMode(NoUntracked | NoSubmodules))
-            != GitClient::StatusUnchanged) {
-        if (QMessageBox::question(
-                    Core::ICore::mainWindow(), tr("Reset"),
-                    tr("All changes in working directory will be discarded. Are you sure?"),
-                    QMessageBox::Yes | QMessageBox::No,
-                    QMessageBox::No) == QMessageBox::No) {
-            return;
-        }
-    }
-    client->reset(workingDir, QLatin1String("--" + resetType), m_currentChange);
+    GitPlugin::client()->reset(
+                sourceWorkingDirectory(), QLatin1String("--" + resetType), m_currentChange);
 }
 
 void GitEditorWidget::cherryPickChange()
 {
-    GitPlugin::instance()->gitClient()->synchronousCherryPick(
-                sourceWorkingDirectory(), m_currentChange);
+    GitPlugin::client()->synchronousCherryPick(sourceWorkingDirectory(), m_currentChange);
 }
 
 void GitEditorWidget::revertChange()
 {
-    GitPlugin::instance()->gitClient()->synchronousRevert(
-                sourceWorkingDirectory(), m_currentChange);
+    GitPlugin::client()->synchronousRevert(sourceWorkingDirectory(), m_currentChange);
 }
 
 void GitEditorWidget::logChange()
 {
-    GitPlugin::instance()->gitClient()->log(
+    GitPlugin::client()->log(
                 sourceWorkingDirectory(), QString(), false, QStringList(m_currentChange));
 }
 
@@ -260,12 +226,11 @@ void GitEditorWidget::applyDiffChunk(const DiffChunk& chunk, bool revert)
     patchFile.write(chunk.chunk);
     patchFile.close();
 
-    GitClient *client = GitPlugin::instance()->gitClient();
-    QStringList args = QStringList() << QLatin1String("--cached");
+    QStringList args = { "--cached" };
     if (revert)
-        args << QLatin1String("--reverse");
+        args << "--reverse";
     QString errorMessage;
-    if (client->synchronousApplyPatch(baseDir, patchFile.fileName(), &errorMessage, args)) {
+    if (GitPlugin::client()->synchronousApplyPatch(baseDir, patchFile.fileName(), &errorMessage, args)) {
         if (errorMessage.isEmpty())
             VcsOutputWindow::append(tr("Chunk successfully staged"));
         else
@@ -304,8 +269,9 @@ void GitEditorWidget::addDiffActions(QMenu *menu, const DiffChunk &chunk)
     });
 }
 
-bool GitEditorWidget::open(QString *errorString, const QString &fileName, const QString &realFileName)
+void GitEditorWidget::aboutToOpen(const QString &fileName, const QString &realFileName)
 {
+    Q_UNUSED(realFileName)
     Core::Id editorId = textDocument()->id();
     if (editorId == Git::Constants::GIT_COMMIT_TEXT_EDITOR_ID
             || editorId == Git::Constants::GIT_REBASE_EDITOR_ID) {
@@ -313,9 +279,8 @@ bool GitEditorWidget::open(QString *errorString, const QString &fileName, const
         const QString gitPath = fi.absolutePath();
         setSource(gitPath);
         textDocument()->setCodec(
-                    GitPlugin::instance()->gitClient()->encoding(gitPath, "i18n.commitEncoding"));
+                    GitPlugin::client()->encoding(gitPath, "i18n.commitEncoding"));
     }
-    return VcsBaseEditorWidget::open(errorString, fileName, realFileName);
 }
 
 QString GitEditorWidget::decorateVersion(const QString &revision) const
@@ -324,19 +289,17 @@ QString GitEditorWidget::decorateVersion(const QString &revision) const
     const QString workingDirectory = fi.absolutePath();
 
     // Format verbose, SHA1 being first token
-    return GitPlugin::instance()->gitClient()->synchronousShortDescription(workingDirectory, revision);
+    return GitPlugin::client()->synchronousShortDescription(workingDirectory, revision);
 }
 
 QStringList GitEditorWidget::annotationPreviousVersions(const QString &revision) const
 {
     QStringList revisions;
     QString errorMessage;
-    GitClient *client = GitPlugin::instance()->gitClient();
     const QFileInfo fi(source());
     const QString workingDirectory = fi.absolutePath();
     // Get the SHA1's of the file.
-    if (!client->synchronousParentRevisions(workingDirectory, QStringList(fi.fileName()),
-                                            revision, &revisions, &errorMessage)) {
+    if (!GitPlugin::client()->synchronousParentRevisions(workingDirectory, revision, &revisions, &errorMessage)) {
         VcsOutputWindow::appendSilently(errorMessage);
         return QStringList();
     }
@@ -345,17 +308,22 @@ QStringList GitEditorWidget::annotationPreviousVersions(const QString &revision)
 
 bool GitEditorWidget::isValidRevision(const QString &revision) const
 {
-    return GitPlugin::instance()->gitClient()->isValidRevision(revision);
+    return GitPlugin::client()->isValidRevision(revision);
 }
 
 void GitEditorWidget::addChangeActions(QMenu *menu, const QString &change)
 {
     m_currentChange = change;
     if (contentType() != OtherContent) {
-        menu->addAction(tr("Cherr&y-Pick Change %1").arg(change), this, SLOT(cherryPickChange()));
-        menu->addAction(tr("Re&vert Change %1").arg(change), this, SLOT(revertChange()));
-        menu->addAction(tr("C&heckout Change %1").arg(change), this, SLOT(checkoutChange()));
-        menu->addAction(tr("&Log for Change %1").arg(change), this, SLOT(logChange()));
+        connect(menu->addAction(tr("Cherr&y-Pick Change %1").arg(change)), &QAction::triggered,
+                this, &GitEditorWidget::cherryPickChange);
+        connect(menu->addAction(tr("Re&vert Change %1").arg(change)), &QAction::triggered,
+                this, &GitEditorWidget::revertChange);
+        connect(menu->addAction(tr("C&heckout Change %1").arg(change)), &QAction::triggered,
+                this, &GitEditorWidget::checkoutChange);
+        connect(menu->addAction(tr("&Log for Change %1").arg(change)), &QAction::triggered,
+                this, &GitEditorWidget::logChange);
+
         QMenu *resetMenu = new QMenu(tr("&Reset to Change %1").arg(change), menu);
         connect(resetMenu->addAction(tr("&Hard")), &QAction::triggered,
                 this, [this]() { resetChange("hard"); });
@@ -391,7 +359,7 @@ QString GitEditorWidget::fileNameForLine(int line) const
     // 7971b6e7 share/qtcreator/dumper/dumper.py   (hjk
     QTextBlock block = document()->findBlockByLineNumber(line - 1);
     QTC_ASSERT(block.isValid(), return source());
-    static QRegExp renameExp(QLatin1String("^" CHANGE_PATTERN "\\s+([^(]+)"));
+    static QRegExp renameExp("^" CHANGE_PATTERN "\\s+([^(]+)");
     if (renameExp.indexIn(block.text()) != -1) {
         const QString fileName = renameExp.cap(1).trimmed();
         if (!fileName.isEmpty())
