@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -41,17 +36,25 @@ RemoteFilterOptions::RemoteFilterOptions(RemoteHelpFilter *filter, QWidget *pare
     , m_filter(filter)
 {
     m_ui.setupUi(this);
+    setWindowTitle(Core::ILocatorFilter::msgConfigureDialogTitle());
+    m_ui.prefixLabel->setText(Core::ILocatorFilter::msgPrefixLabel());
+    m_ui.prefixLabel->setToolTip(Core::ILocatorFilter::msgPrefixToolTip());
+    m_ui.includeByDefault->setText(Core::ILocatorFilter::msgIncludeByDefault());
+    m_ui.includeByDefault->setToolTip(Core::ILocatorFilter::msgIncludeByDefaultToolTip());
     m_ui.shortcutEdit->setText(m_filter->shortcutString());
-    m_ui.limitCheck->setChecked(!m_filter->isIncludedByDefault());
+    m_ui.includeByDefault->setChecked(m_filter->isIncludedByDefault());
     foreach (const QString &url, m_filter->remoteUrls()) {
         QListWidgetItem *item = new QListWidgetItem(url);
         m_ui.listWidget->addItem(item);
         item->setFlags(item->flags() | Qt::ItemIsEditable);
     }
 
-    connect(m_ui.add, SIGNAL(clicked()), this, SLOT(addNewItem()));
-    connect(m_ui.remove, SIGNAL(clicked()), this, SLOT(removeItem()));
-    connect(m_ui.listWidget, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), SLOT(updateRemoveButton()));
+    connect(m_ui.add, &QPushButton::clicked,
+            this, &RemoteFilterOptions::addNewItem);
+    connect(m_ui.remove, &QPushButton::clicked,
+            this, &RemoteFilterOptions::removeItem);
+    connect(m_ui.listWidget, &QListWidget::currentItemChanged,
+            this, &RemoteFilterOptions::updateRemoveButton);
     updateRemoveButton();
 }
 
@@ -85,13 +88,13 @@ RemoteHelpFilter::RemoteHelpFilter()
     setId("RemoteHelpFilter");
     setDisplayName(tr("Web Search"));
     setIncludedByDefault(false);
-    setShortcutString(QLatin1String("r"));
+    setShortcutString("r");
 
-    m_remoteUrls.append(QLatin1String("https://www.bing.com/search?q=%1"));
-    m_remoteUrls.append(QLatin1String("https://www.google.com/search?q=%1"));
-    m_remoteUrls.append(QLatin1String("https://search.yahoo.com/search?p=%1"));
-    m_remoteUrls.append(QLatin1String("https://www.cplusplus.com/reference/stl/%1"));
-    m_remoteUrls.append(QLatin1String("https://en.wikipedia.org/w/index.php?search=%1"));
+    m_remoteUrls.append("https://www.bing.com/search?q=%1");
+    m_remoteUrls.append("https://www.google.com/search?q=%1");
+    m_remoteUrls.append("https://search.yahoo.com/search?p=%1");
+    m_remoteUrls.append("https://www.cplusplus.com/reference/stl/%1");
+    m_remoteUrls.append("https://en.wikipedia.org/w/index.php?search=%1");
 }
 
 RemoteHelpFilter::~RemoteHelpFilter()
@@ -160,7 +163,7 @@ bool RemoteHelpFilter::openConfigDialog(QWidget *parent, bool &needsRefresh)
     if (optionsDialog.exec() == QDialog::Accepted) {
         QMutexLocker lock(&m_mutex); Q_UNUSED(lock)
         m_remoteUrls.clear();
-        setIncludedByDefault(!optionsDialog.m_ui.limitCheck->isChecked());
+        setIncludedByDefault(optionsDialog.m_ui.includeByDefault->isChecked());
         setShortcutString(optionsDialog.m_ui.shortcutEdit->text().trimmed());
         for (int i = 0; i < optionsDialog.m_ui.listWidget->count(); ++i)
             m_remoteUrls.append(optionsDialog.m_ui.listWidget->item(i)->text());
