@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef TEXTBROWSERHELPVIEWER_H
-#define TEXTBROWSERHELPVIEWER_H
+#pragma once
 
 #include "centralwidget.h"
 #include "helpviewer.h"
@@ -47,19 +41,19 @@ class TextBrowserHelpViewer : public HelpViewer
     Q_OBJECT
 
 public:
-    explicit TextBrowserHelpViewer(qreal zoom, QWidget *parent = 0);
+    explicit TextBrowserHelpViewer(QWidget *parent = 0);
     ~TextBrowserHelpViewer();
 
     QFont viewerFont() const;
     void setViewerFont(const QFont &font);
 
     qreal scale() const;
+    void setScale(qreal scale);
 
     QString title() const;
 
     QUrl source() const;
     void setSource(const QUrl &url);
-    void scrollToAnchor(const QString &anchor);
 
     void setHtml(const QString &html);
 
@@ -73,7 +67,6 @@ public:
     bool findText(const QString &text, Core::FindFlags flags,
                   bool incremental, bool fromSearch, bool *wrapped = 0);
 
-public slots:
     void scaleUp();
     void scaleDown();
     void resetScale();
@@ -83,11 +76,8 @@ public slots:
     void backward();
     void print(QPrinter *printer);
 
-private slots:
-    void goToHistoryItem();
-
 private:
-    QVariant loadResource(int type, const QUrl &name);
+    void goToHistoryItem();
 
     TextBrowserHelpWidget *m_textBrowser;
 };
@@ -97,13 +87,15 @@ class TextBrowserHelpWidget : public QTextBrowser
     Q_OBJECT
 
 public:
-    TextBrowserHelpWidget(int zoom, TextBrowserHelpViewer *parent);
+    TextBrowserHelpWidget(TextBrowserHelpViewer *parent);
 
     QVariant loadResource(int type, const QUrl &name);
 
     void scaleUp();
     void scaleDown();
 
+    void setSource(const QUrl &name);
+
 protected:
     void contextMenuEvent(QContextMenuEvent *event);
     bool eventFilter(QObject *obj, QEvent *event);
@@ -115,14 +107,12 @@ private:
     QString linkAt(const QPoint& pos);
     void openLink(const QUrl &url, bool newPage);
 
-public:
     int zoomCount;
     bool forceFont;
     bool m_openInNewPageActionVisible;
     TextBrowserHelpViewer *m_parent;
+    friend class Help::Internal::TextBrowserHelpViewer;
 };
 
 }   // namespace Internal
 }   // namespace Help
-
-#endif // TEXTBROWSERHELPVIEWER_H
