@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,21 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -40,9 +36,9 @@
 #include <projectexplorer/projectexplorerconstants.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/taskhub.h>
-#include <qmakeprojectmanager/qmakebuildconfiguration.h>
-#include <qmakeprojectmanager/qmakeproject.h>
-#include <qmakeprojectmanager/qmakenodes.h>
+#include <projectexplorer/kitmanager.h>
+#include <projectexplorer/kitinformation.h>
+#include <projectexplorer/devicesupport/devicemanager.h>
 
 #include <qtsupport/qtkitinformation.h>
 
@@ -54,21 +50,20 @@
 #define ASSERT_STATE(state) ASSERT_STATE_GENERIC(State, state, m_state)
 
 using namespace ProjectExplorer;
-using namespace QmakeProjectManager;
 
 namespace Ios {
 namespace Internal {
 
 const Core::Id IosDeployStep::Id("Qt4ProjectManager.IosDeployStep");
 
-IosDeployStep::IosDeployStep(ProjectExplorer::BuildStepList *parent)
+IosDeployStep::IosDeployStep(BuildStepList *parent)
     : BuildStep(parent, Id)
     , m_expectFail(false)
 {
     ctor();
 }
 
-IosDeployStep::IosDeployStep(ProjectExplorer::BuildStepList *parent,
+IosDeployStep::IosDeployStep(BuildStepList *parent,
     IosDeployStep *other)
     : BuildStep(parent, other)
     , m_expectFail(false)
@@ -76,25 +71,36 @@ IosDeployStep::IosDeployStep(ProjectExplorer::BuildStepList *parent,
     ctor();
 }
 
-IosDeployStep::~IosDeployStep() { }
-
 void IosDeployStep::ctor()
 {
     m_toolHandler = 0;
     m_transferStatus = NoTransfer;
-    m_device = ProjectExplorer::DeviceKitInformation::device(target()->kit());
-    const QString devName = m_device.isNull() ? IosDevice::name() : m_device->displayName();
-    setDefaultDisplayName(tr("Deploy to %1").arg(devName));
+    cleanup();
+    updateDisplayNames();
+    connect(DeviceManager::instance(), &DeviceManager::updated,
+            this, &IosDeployStep::updateDisplayNames);
+    connect(target(), &Target::kitChanged,
+            this, &IosDeployStep::updateDisplayNames);
 }
 
-bool IosDeployStep::init()
+void IosDeployStep::updateDisplayNames()
 {
-    QTC_CHECK(m_transferStatus == NoTransfer);
-    m_device = ProjectExplorer::DeviceKitInformation::device(target()->kit());
+    IDevice::ConstPtr dev =
+            DeviceKitInformation::device(target()->kit());
+    const QString devName = dev.isNull() ? IosDevice::name() : dev->displayName();
+    setDefaultDisplayName(tr("Deploy to %1").arg(devName));
+    setDisplayName(tr("Deploy to %1").arg(devName));
+}
+
+bool IosDeployStep::init(QList<const BuildStep *> &earlierSteps)
+{
+    Q_UNUSED(earlierSteps);
+    QTC_ASSERT(m_transferStatus == NoTransfer, return false);
+    m_device = DeviceKitInformation::device(target()->kit());
     IosRunConfiguration * runConfig = qobject_cast<IosRunConfiguration *>(
                 this->target()->activeRunConfiguration());
     QTC_ASSERT(runConfig, return false);
-    m_bundlePath = runConfig->bundleDir().toString();
+    m_bundlePath = runConfig->bundleDirectory().toString();
     if (m_device.isNull()) {
         emit addOutput(tr("Error: no device available, deploy failed."),
                        BuildStep::ErrorMessageOutput);
@@ -111,25 +117,24 @@ void IosDeployStep::run(QFutureInterface<bool> &fi)
         if (iossimulator().isNull())
             TaskHub::addTask(Task::Error, tr("Deployment failed. No iOS device found."),
                              ProjectExplorer::Constants::TASK_CATEGORY_DEPLOYMENT);
-        m_futureInterface.reportResult(!iossimulator().isNull());
+        reportRunResult(m_futureInterface, !iossimulator().isNull());
         cleanup();
-        emit finished();
         return;
     }
     m_transferStatus = TransferInProgress;
     QTC_CHECK(m_toolHandler == 0);
-    m_toolHandler = new IosToolHandler(IosDeviceType::IosDevice, this);
+    m_toolHandler = new IosToolHandler(IosDeviceType(IosDeviceType::IosDevice), this);
     m_futureInterface.setProgressRange(0, 200);
     m_futureInterface.setProgressValueAndText(0, QLatin1String("Transferring application"));
     m_futureInterface.reportStarted();
-    connect(m_toolHandler, SIGNAL(isTransferringApp(Ios::IosToolHandler*,QString,QString,int,int,QString)),
-            SLOT(handleIsTransferringApp(Ios::IosToolHandler*,QString,QString,int,int,QString)));
-    connect(m_toolHandler, SIGNAL(didTransferApp(Ios::IosToolHandler*,QString,QString,Ios::IosToolHandler::OpStatus)),
-            SLOT(handleDidTransferApp(Ios::IosToolHandler*,QString,QString,Ios::IosToolHandler::OpStatus)));
-    connect(m_toolHandler, SIGNAL(finished(Ios::IosToolHandler*)),
-            SLOT(handleFinished(Ios::IosToolHandler*)));
-    connect(m_toolHandler, SIGNAL(errorMsg(Ios::IosToolHandler*,QString)),
-            SLOT(handleErrorMsg(Ios::IosToolHandler*,QString)));
+    connect(m_toolHandler, &IosToolHandler::isTransferringApp,
+            this, &IosDeployStep::handleIsTransferringApp);
+    connect(m_toolHandler, &IosToolHandler::didTransferApp,
+            this, &IosDeployStep::handleDidTransferApp);
+    connect(m_toolHandler, &IosToolHandler::finished,
+            this, &IosDeployStep::handleFinished);
+    connect(m_toolHandler, &IosToolHandler::errorMsg,
+            this, &IosDeployStep::handleErrorMsg);
     checkProvisioningProfile();
     m_toolHandler->requestTransferApp(appBundle(), deviceId());
 }
@@ -170,10 +175,10 @@ void IosDeployStep::handleDidTransferApp(IosToolHandler *handler, const QString
         m_transferStatus = TransferFailed;
         if (!m_expectFail)
             TaskHub::addTask(Task::Error,
-                             tr("Deployment failed. The settings in the Organizer window of Xcode might be incorrect."),
+                             tr("Deployment failed. The settings in the Devices window of Xcode might be incorrect."),
                              ProjectExplorer::Constants::TASK_CATEGORY_DEPLOYMENT);
     }
-    m_futureInterface.reportResult(status == IosToolHandler::Success);
+    reportRunResult(m_futureInterface, status == IosToolHandler::Success);
 }
 
 void IosDeployStep::handleFinished(IosToolHandler *handler)
@@ -183,7 +188,7 @@ void IosDeployStep::handleFinished(IosToolHandler *handler)
         m_transferStatus = TransferFailed;
         TaskHub::addTask(Task::Error, tr("Deployment failed."),
                          ProjectExplorer::Constants::TASK_CATEGORY_DEPLOYMENT);
-        m_futureInterface.reportResult(false);
+        reportRunResult(m_futureInterface, false);
         break;
     case NoTransfer:
     case TransferOk:
@@ -193,7 +198,6 @@ void IosDeployStep::handleFinished(IosToolHandler *handler)
     cleanup();
     handler->deleteLater();
     // move it when result is reported? (would need care to avoid problems with concurrent runs)
-    emit finished();
 }
 
 void IosDeployStep::handleErrorMsg(IosToolHandler *handler, const QString &msg)
@@ -213,12 +217,12 @@ BuildStepConfigWidget *IosDeployStep::createConfigWidget()
 
 bool IosDeployStep::fromMap(const QVariantMap &map)
 {
-    return ProjectExplorer::BuildStep::fromMap(map);
+    return BuildStep::fromMap(map);
 }
 
 QVariantMap IosDeployStep::toMap() const
 {
-    QVariantMap map = ProjectExplorer::BuildStep::toMap();
+    QVariantMap map = BuildStep::toMap();
     return map;
 }
 
@@ -256,7 +260,7 @@ void IosDeployStep::checkProvisioningProfile()
 
     // the file is a signed plist stored in DER format
     // we simply search for start and end of the plist instead of decoding the DER payload
-    if (!provisioningFilePath.toFileInfo().exists())
+    if (!provisioningFilePath.exists())
         return;
     QFile provisionFile(provisioningFilePath.toString());
     if (!provisionFile.open(QIODevice::ReadOnly))
