@@ -1,7 +1,7 @@
-/**************************************************************************
+/****************************************************************************
 **
-** Copyright (c) 2014 Nicolas Arnaud-Cormos
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 Nicolas Arnaud-Cormos
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -43,10 +38,7 @@
 
 #include <QAction>
 #include <QEvent>
-#include <QObject>
-#include <QShortcut>
 #include <QSignalMapper>
-#include <QStringList>
 
 using namespace Core;
 
@@ -56,14 +48,10 @@ namespace Internal {
 static const char EVENTNAME[] = "Action";
 static quint8 ACTIONNAME = 0;
 
-ActionMacroHandler::ActionMacroHandler():
-    m_mapper(new QSignalMapper(this))
+ActionMacroHandler::ActionMacroHandler()
 {
-    connect(m_mapper, SIGNAL(mapped(QString)),
-            this, SLOT(addActionEvent(QString)));
-
-    connect(ActionManager::instance(), SIGNAL(commandAdded(QString)),
-            this, SLOT(addCommand(QString)));
+    connect(ActionManager::instance(), &ActionManager::commandAdded,
+            this, &ActionMacroHandler::addCommand);
 
     // Register all existing scriptable actions
     QList<Command *> commands = ActionManager::commands();
@@ -88,37 +76,29 @@ bool ActionMacroHandler::executeEvent(const MacroEvent &macroEvent)
     return true;
 }
 
-void ActionMacroHandler::addActionEvent(const QString &name)
-{
-    if (!isRecording())
-        return;
-
-    const Id id = Id::fromString(name);
-    const Command *command = ActionManager::command(id);
-    if (command->isScriptable(command->context())) {
-        MacroEvent e;
-        e.setId(EVENTNAME);
-        e.setValue(ACTIONNAME, id.toSetting());
-        addMacroEvent(e);
-    }
-}
-
 void ActionMacroHandler::registerCommand(Id id)
 {
     if (!m_commandIds.contains(id)) {
         m_commandIds.insert(id);
         const Command *command = ActionManager::command(id);
         if (QAction *action = command->action()) {
-            connect(action, SIGNAL(triggered()), m_mapper, SLOT(map()));
-            m_mapper->setMapping(action, id.toString());
-            return;
+            connect(action, &QAction::triggered, this, [this, id, command]() {
+                if (!isRecording())
+                    return;
+
+                if (command->isScriptable(command->context())) {
+                    MacroEvent e;
+                    e.setId(EVENTNAME);
+                    e.setValue(ACTIONNAME, id.toSetting());
+                    addMacroEvent(e);
+                }
+            });
         }
     }
 }
 
-void ActionMacroHandler::addCommand(const QString &name)
+void ActionMacroHandler::addCommand(Id id)
 {
-    const Id id = Id::fromString(name);
     const Command *command = ActionManager::command(id);
     if (command->isScriptable())
         registerCommand(id);
