@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef KITMANAGERCONFIGWIDGET_H
-#define KITMANAGERCONFIGWIDGET_H
+#pragma once
 
 #include "kitconfigwidget.h"
 
@@ -53,7 +47,7 @@ class KitManagerConfigWidget : public QWidget
 
 public:
     explicit KitManagerConfigWidget(Kit *k);
-    ~KitManagerConfigWidget();
+    ~KitManagerConfigWidget() override;
 
     QString displayName() const;
 
@@ -63,36 +57,37 @@ public:
     bool isValid() const;
     bool hasWarning() const;
     QString validityMessage() const;
-    void addConfigWidget(ProjectExplorer::KitConfigWidget *widget);
+    void addConfigWidget(KitConfigWidget *widget);
     void makeStickySubWidgetsReadOnly();
 
     Kit *workingCopy() const;
-    bool configures(ProjectExplorer::Kit *k) const;
+    bool configures(Kit *k) const;
     void setIsDefaultKit(bool d);
     bool isDefaultKit() const;
     void removeKit();
     void updateVisibility();
 
+    void setHasUniqueName(bool unique);
+
 signals:
     void dirty();
+    void isAutoDetectedChanged();
 
-private slots:
+private:
     void setIcon();
     void setDisplayName();
     void setFileSystemFriendlyName();
     void workingCopyWasUpdated(ProjectExplorer::Kit *k);
     void kitWasUpdated(ProjectExplorer::Kit *k);
-    void updateMutableState();
 
-private:
     enum LayoutColumns {
         LabelColumn,
         WidgetColumn,
         ButtonColumn
     };
 
+    void showEvent(QShowEvent *event) override;
     QLabel *createLabel(const QString &name, const QString &toolTip);
-    void paintEvent(QPaintEvent *ev);
 
     QGridLayout *m_layout;
     QToolButton *m_iconButton;
@@ -102,13 +97,12 @@ private:
     QList<QLabel *> m_labels;
     Kit *m_kit;
     Kit *m_modifiedKit;
-    bool m_isDefaultKit;
-    bool m_fixingKit;
-    QPixmap m_background;
+    bool m_isDefaultKit = false;
+    bool m_fixingKit = false;
+    bool m_hasUniqueName = true;
     QList<QAction *> m_actions;
+    mutable QString m_cachedDisplayName;
 };
 
 } // namespace Internal
 } // namespace ProjectExplorer
-
-#endif // KITMANAGERCONFIGWIDGET_H
