@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -47,7 +42,7 @@
 #include <QIcon>
 #include <QStyle>
 
-using namespace ProjectExplorer;
+namespace ProjectExplorer {
 
 /*!
   \class ProjectExplorer::Node
@@ -64,20 +59,14 @@ using namespace ProjectExplorer;
   \sa ProjectExplorer::NodesWatcher, ProjectExplorer::NodesVisitor
 */
 
-Node::Node(NodeType nodeType, const Utils::FileName &filePath, int line)
-        : m_nodeType(nodeType),
-          m_line(line),
-          m_projectNode(0),
-          m_folderNode(0),
-          m_path(filePath)
-{
-
-}
+Node::Node(NodeType nodeType, const Utils::FileName &filePath, int line) :
+    m_nodeType(nodeType),
+    m_line(line),
+    m_filePath(filePath)
+{ }
 
 Node::~Node()
-{
-
-}
+{ }
 
 void Node::emitNodeSortKeyAboutToChange()
 {
@@ -91,40 +80,13 @@ void Node::emitNodeSortKeyChanged()
         ProjectTree::instance()->emitNodeSortKeyChanged(this);
 }
 
-/*!
- * The path of the file representing this node.
- *
- * This function does not emit any signals. That has to be done by the calling
- * class.
- */
-void Node::setPath(const Utils::FileName &path)
+void Node::setAbsoluteFilePathAndLine(const Utils::FileName &path, int line)
 {
-    if (m_path == path)
+    if (m_filePath == path && m_line == line)
         return;
 
     emitNodeSortKeyAboutToChange();
-    m_path = path;
-    emitNodeSortKeyChanged();
-    emitNodeUpdated();
-}
-
-void Node::setLine(int line)
-{
-    if (m_line == line)
-        return;
-    emitNodeSortKeyAboutToChange();
-    m_line = line;
-    emitNodeSortKeyChanged();
-    emitNodeUpdated();
-}
-
-void Node::setPathAndLine(const Utils::FileName &path, int line)
-{
-    if (m_path == path
-            && m_line == line)
-        return;
-    emitNodeSortKeyAboutToChange();
-    m_path = path;
+    m_filePath = path;
     m_line = line;
     emitNodeSortKeyChanged();
     emitNodeUpdated();
@@ -155,9 +117,9 @@ FolderNode *Node::parentFolderNode() const
 /*!
   The path of the file or folder in the filesystem the node represents.
   */
-const Utils::FileName &Node::path() const
+const Utils::FileName &Node::filePath() const
 {
-    return m_path;
+    return m_filePath;
 }
 
 int Node::line() const
@@ -167,12 +129,12 @@ int Node::line() const
 
 QString Node::displayName() const
 {
-    return path().fileName();
+    return filePath().fileName();
 }
 
 QString Node::tooltip() const
 {
-    return path().toUserOutput();
+    return filePath().toUserOutput();
 }
 
 bool Node::isEnabled() const
@@ -200,22 +162,22 @@ void Node::emitNodeUpdated()
 
 FileNode *Node::asFileNode()
 {
-    return 0;
+    return nullptr;
 }
 
 FolderNode *Node::asFolderNode()
 {
-    return 0;
+    return nullptr;
 }
 
 ProjectNode *Node::asProjectNode()
 {
-    return 0;
+    return nullptr;
 }
 
 SessionNode *Node::asSessionNode()
 {
-    return 0;
+    return nullptr;
 }
 
 void Node::setParentFolderNode(FolderNode *parentFolder)
@@ -235,12 +197,10 @@ void Node::setParentFolderNode(FolderNode *parentFolder)
 
 FileNode::FileNode(const Utils::FileName &filePath,
                    const FileType fileType,
-                   bool generated, int line)
-        : Node(FileNodeType, filePath, line),
-          m_fileType(fileType),
-          m_generated(generated)
-{
-}
+                   bool generated, int line) : Node(FileNodeType, filePath, line),
+    m_fileType(fileType),
+    m_generated(generated)
+{ }
 
 FileType FileNode::fileType() const
 {
@@ -336,6 +296,11 @@ void FolderNode::setIcon(const QIcon &icon)
     m_icon = icon;
 }
 
+QString FolderNode::addFileFilter() const
+{
+    return parentFolderNode()->addFileFilter();
+}
+
 bool FolderNode::addFiles(const QStringList &filePaths, QStringList *notAdded)
 {
     if (projectNode())
@@ -405,10 +370,7 @@ void FolderNode::addFileNodes(const QList<FileNode *> &files)
             // empty list or greater then last node
             m_fileNodes.append(file);
         } else {
-            QList<FileNode *>::iterator it
-                    = qLowerBound(m_fileNodes.begin(),
-                                  m_fileNodes.end(),
-                                  file);
+            auto it = qLowerBound(m_fileNodes.begin(), m_fileNodes.end(), file);
             m_fileNodes.insert(it, file);
         }
     }
@@ -436,8 +398,8 @@ void FolderNode::removeFileNodes(const QList<FileNode *> &files)
 
     ProjectTree::instance()->emitFilesAboutToBeRemoved(this, toRemove);
 
-    QList<FileNode*>::const_iterator toRemoveIter = toRemove.constBegin();
-    QList<FileNode*>::iterator filesIter = m_fileNodes.begin();
+    auto toRemoveIter = toRemove.constBegin();
+    auto filesIter = m_fileNodes.begin();
     for (; toRemoveIter != toRemove.constEnd(); ++toRemoveIter) {
         while (*filesIter != *toRemoveIter) {
             ++filesIter;
@@ -476,10 +438,7 @@ void FolderNode::addFolderNodes(const QList<FolderNode*> &subFolders)
             m_subFolderNodes.append(folder);
         } else {
             // Binary Search for insertion point
-            QList<FolderNode*>::iterator it
-                    = qLowerBound(m_subFolderNodes.begin(),
-                                  m_subFolderNodes.end(),
-                                  folder);
+            auto it = qLowerBound(m_subFolderNodes.begin(), m_subFolderNodes.end(), folder);
             m_subFolderNodes.insert(it, folder);
         }
 
@@ -509,8 +468,8 @@ void FolderNode::removeFolderNodes(const QList<FolderNode*> &subFolders)
 
     ProjectTree::instance()->emitFoldersAboutToBeRemoved(this, toRemove);
 
-    QList<FolderNode*>::const_iterator toRemoveIter = toRemove.constBegin();
-    QList<FolderNode*>::iterator folderIter = m_subFolderNodes.begin();
+    auto toRemoveIter = toRemove.constBegin();
+    auto folderIter = m_subFolderNodes.begin();
     for (; toRemoveIter != toRemove.constEnd(); ++toRemoveIter) {
         QTC_ASSERT((*toRemoveIter)->nodeType() != ProjectNodeType,
                    qDebug("project nodes have to be removed via removeProjectNodes"));
@@ -547,14 +506,10 @@ bool FolderNode::showInSimpleTree() const
 
   \sa ProjectExplorer::FileNode, ProjectExplorer::ProjectNode
 */
-VirtualFolderNode::VirtualFolderNode(const Utils::FileName &folderPath, int priority)
-    : FolderNode(folderPath, VirtualFolderNodeType), m_priority(priority)
-{
-}
-
-VirtualFolderNode::~VirtualFolderNode()
-{
-}
+VirtualFolderNode::VirtualFolderNode(const Utils::FileName &folderPath, int priority) :
+    FolderNode(folderPath, VirtualFolderNodeType),
+    m_priority(priority)
+{ }
 
 int VirtualFolderNode::priority() const
 {
@@ -574,8 +529,8 @@ int VirtualFolderNode::priority() const
 /*!
   Creates an uninitialized project node object.
   */
-ProjectNode::ProjectNode(const Utils::FileName &projectFilePath)
-        : FolderNode(projectFilePath, ProjectNodeType)
+ProjectNode::ProjectNode(const Utils::FileName &projectFilePath) :
+    FolderNode(projectFilePath, ProjectNodeType)
 {
     // project node "manages" itself
     setProjectNode(this);
@@ -584,7 +539,7 @@ ProjectNode::ProjectNode(const Utils::FileName &projectFilePath)
 
 QString ProjectNode::vcsTopic() const
 {
-    const QString dir = path().toFileInfo().absolutePath();
+    const QString dir = filePath().toFileInfo().absolutePath();
 
     if (Core::IVersionControl *const vc =
             Core::VcsManager::findVersionControlForDirectory(dir))
@@ -718,9 +673,9 @@ void ProjectNode::removeProjectNodes(const QList<ProjectNode*> &subProjects)
 
         ProjectTree::instance()->emitFoldersAboutToBeRemoved(this, toRemove);
 
-        QList<FolderNode*>::const_iterator toRemoveIter = toRemove.constBegin();
-        QList<FolderNode*>::iterator folderIter = m_subFolderNodes.begin();
-        QList<ProjectNode*>::iterator projectIter = m_subProjectNodes.begin();
+        auto toRemoveIter = toRemove.constBegin();
+        auto folderIter = m_subFolderNodes.begin();
+        auto projectIter = m_subProjectNodes.begin();
         for (; toRemoveIter != toRemove.constEnd(); ++toRemoveIter) {
             while (*projectIter != *toRemoveIter) {
                 ++projectIter;
@@ -751,10 +706,9 @@ ProjectNode *ProjectNode::asProjectNode()
   \class ProjectExplorer::SessionNode
 */
 
-SessionNode::SessionNode()
-    : FolderNode(Utils::FileName::fromString(QLatin1String("session")), SessionNodeType)
-{
-}
+SessionNode::SessionNode() :
+    FolderNode(Utils::FileName::fromString(QLatin1String("session")), SessionNodeType)
+{ }
 
 QList<ProjectAction> SessionNode::supportedActions(Node *node) const
 {
@@ -791,6 +745,11 @@ QList<ProjectNode*> SessionNode::projectNodes() const
     return m_projectNodes;
 }
 
+QString SessionNode::addFileFilter() const
+{
+    return QLatin1String("*.c; *.cc; *.cpp; *.cp; *.cxx; *.c++; *.h; *.hh; *.hpp; *.hxx;");
+}
+
 void SessionNode::addProjectNodes(const QList<ProjectNode*> &projectNodes)
 {
     if (!projectNodes.isEmpty()) {
@@ -826,9 +785,9 @@ void SessionNode::removeProjectNodes(const QList<ProjectNode*> &projectNodes)
 
         ProjectTree::instance()->emitFoldersAboutToBeRemoved(this, toRemove);
 
-        QList<FolderNode*>::const_iterator toRemoveIter = toRemove.constBegin();
-        QList<FolderNode*>::iterator folderIter = m_subFolderNodes.begin();
-        QList<ProjectNode*>::iterator projectIter = m_projectNodes.begin();
+        auto toRemoveIter = toRemove.constBegin();
+        auto folderIter = m_subFolderNodes.begin();
+        auto projectIter = m_projectNodes.begin();
         for (; toRemoveIter != toRemove.constEnd(); ++toRemoveIter) {
             while (*projectIter != *toRemoveIter) {
                 ++projectIter;
@@ -847,3 +806,5 @@ void SessionNode::removeProjectNodes(const QList<ProjectNode*> &projectNodes)
         ProjectTree::instance()->emitFoldersRemoved(this);
     }
 }
+
+} // namespace ProjectExplorer
