@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef PROJECTWIZARDPAGE_H
-#define PROJECTWIZARDPAGE_H
+#pragma once
 
 #include "projectnodes.h"
 
@@ -59,8 +53,8 @@ class ProjectWizardPage : public Utils::WizardPage
     Q_OBJECT
 
 public:
-    explicit ProjectWizardPage(QWidget *parent = 0);
-    virtual ~ProjectWizardPage();
+    explicit ProjectWizardPage(QWidget *parent = nullptr);
+    ~ProjectWizardPage() override;
 
     FolderNode *currentNode() const;
 
@@ -79,20 +73,18 @@ public:
                                Core::IWizardFactory::WizardKind kind,
                                ProjectAction action);
 
+    void initializeVersionControls();
+    void setProjectUiVisible(bool visible);
+
 signals:
     void projectNodeChanged();
     void versionControlChanged(int);
 
-public slots:
-    void initializeVersionControls();
-    void setProjectUiVisible(bool visible);
-
-private slots:
+private:
     void projectChanged(int);
     void manageVcs();
     void hideVersionControlUiElements();
 
-private:
     void setAdditionalInfo(const QString &text);
     void setAddingSubProject(bool addingSubProject);
     void setModel(Utils::TreeModel *model);
@@ -103,14 +95,12 @@ private:
 
     Ui::WizardPage *m_ui;
     QStringList m_projectToolTips;
-    Utils::TreeModel *m_model;
+    Utils::TreeModel *m_model = nullptr;
 
     QList<Core::IVersionControl*> m_activeVersionControls;
     QString m_commonDirectory;
-    bool m_repositoryExists;
+    bool m_repositoryExists = false;
 };
 
 } // namespace Internal
 } // namespace ProjectExplorer
-
-#endif // PROJECTWIZARDPAGE_H
