@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef DESKTOPQMAKERUNCONFIGURATION_H
-#define DESKTOPQMAKERUNCONFIGURATION_H
+#pragma once
 
 #include <qmakeprojectmanager/qmakerunconfigurationfactory.h>
 
@@ -87,7 +86,7 @@ signals:
     // Note: These signals might not get emitted for every change!
     void effectiveTargetInformationChanged();
 
-private slots:
+private:
     void proFileUpdated(QmakeProjectManager::QmakeProFileNode *pro, bool success, bool parseInProgress);
     void proFileEvaluated();
 
@@ -171,5 +170,3 @@ private:
 
 } // namespace Internal
 } // namespace QmakeProjectManager
-
-#endif // DESKTOPQMAKERUNCONFIGURATION_H
