@@ -26,15 +26,17 @@
 #include "qmakestep.h"
 #include "ui_qmakestep.h"
 
-#include "qmakeparser.h"
+#include "makestep.h"
 #include "qmakebuildconfiguration.h"
-#include "qmakeproject.h"
-#include "qmakeprojectmanagerconstants.h"
 #include "qmakekitinformation.h"
 #include "qmakenodes.h"
+#include "qmakeparser.h"
+#include "qmakeproject.h"
+#include "qmakeprojectmanagerconstants.h"
 
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/buildsteplist.h>
+#include <projectexplorer/gnumakeparser.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/toolchain.h>
@@ -83,8 +85,8 @@ QMakeStep::QMakeStep(BuildStepList *bsl, Core::Id id) :
 
 QMakeStep::QMakeStep(BuildStepList *bsl, QMakeStep *bs) :
     AbstractProcessStep(bsl, bs),
-    m_forced(bs->m_forced),
     m_userArgs(bs->m_userArgs),
+    m_forced(bs->m_forced),
     m_linkQmlDebuggingLibrary(bs->m_linkQmlDebuggingLibrary),
     m_useQtQuickCompiler(bs->m_useQtQuickCompiler),
     m_separateDebugInfo(bs->m_separateDebugInfo)
@@ -96,6 +98,13 @@ void QMakeStep::ctor()
 {
     //: QMakeStep default display name
     setDefaultDisplayName(tr("qmake"));
+
+    connect(&m_inputWatcher, &QFutureWatcher<bool>::canceled,
+            this, [this]() {
+                if (m_commandFuture)
+                    m_commandFuture->cancel();
+            });
+    connect(&m_commandWatcher, &QFutureWatcher<bool>::finished, this, &QMakeStep::runNextCommand);
 }
 
 QmakeBuildConfiguration *QMakeStep::qmakeBuildConfiguration() const
@@ -110,8 +119,9 @@ QmakeBuildConfiguration *QMakeStep::qmakeBuildConfiguration() const
 /// config arguemnts
 /// moreArguments
 /// user arguments
-QString QMakeStep::allArguments(bool shorted)
+QString QMakeStep::allArguments(const BaseQtVersion *v, bool shorted) const
 {
+    QTC_ASSERT(v, return QString());
     QmakeBuildConfiguration *bc = qmakeBuildConfiguration();
     QStringList arguments;
     if (bc->subNodeBuild())
@@ -121,7 +131,8 @@ QString QMakeStep::allArguments(bool shorted)
     else
         arguments << project()->projectFilePath().toUserOutput();
 
-    arguments << QLatin1String("-r");
+    if (v->qtVersion() < QtVersionNumber(5, 0, 0))
+        arguments << QLatin1String("-r");
     bool userProvidedMkspec = false;
     for (QtcProcess::ConstArgIterator ait(m_userArgs); ait.next(); ) {
         if (ait.value() == QLatin1String("-spec")) {
@@ -146,7 +157,7 @@ QString QMakeStep::allArguments(bool shorted)
     return args;
 }
 
-QMakeStepConfig QMakeStep::deducedArguments()
+QMakeStepConfig QMakeStep::deducedArguments() const
 {
     ProjectExplorer::Kit *kit = target()->kit();
     QMakeStepConfig config;
@@ -156,7 +167,7 @@ QMakeStepConfig QMakeStep::deducedArguments()
     if (tc)
         targetAbi = tc->targetAbi();
 
-    QtSupport::BaseQtVersion *version = QtSupport::QtKitInformation::qtVersion(target()->kit());
+    BaseQtVersion *version = QtKitInformation::qtVersion(target()->kit());
 
     config.archConfig = QMakeStepConfig::targetArchFor(targetAbi, version);
     config.osType = QMakeStepConfig::osTypeFor(targetAbi, version);
@@ -175,55 +186,65 @@ QMakeStepConfig QMakeStep::deducedArguments()
 
 bool QMakeStep::init(QList<const BuildStep *> &earlierSteps)
 {
-    QmakeBuildConfiguration *qt4bc = qmakeBuildConfiguration();
-    const QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(target()->kit());
+    if (m_commandFuture)
+        return false;
+
+    QmakeBuildConfiguration *qmakeBc = qmakeBuildConfiguration();
+    const BaseQtVersion *qtVersion = QtKitInformation::qtVersion(target()->kit());
 
     if (!qtVersion)
         return false;
 
-    QString args = allArguments();
     QString workingDirectory;
 
-    if (qt4bc->subNodeBuild())
-        workingDirectory = qt4bc->subNodeBuild()->buildDir();
+    if (qmakeBc->subNodeBuild())
+        workingDirectory = qmakeBc->subNodeBuild()->buildDir();
     else
-        workingDirectory = qt4bc->buildDirectory().toString();
+        workingDirectory = qmakeBc->buildDirectory().toString();
 
-    FileName program = qtVersion->qmakeCommand();
+    m_qmakeExecutable = qtVersion->qmakeCommand().toString();
+    m_qmakeArguments = allArguments(qtVersion);
+    m_runMakeQmake = (qtVersion->qtVersion() >= QtVersionNumber(5, 0 ,0));
+    if (m_runMakeQmake) {
+        m_makeExecutable = makeCommand();
+        if (m_makeExecutable.isEmpty())
+            return false;
+        m_makeArguments = makeArguments();
+    } else {
+        m_makeExecutable.clear();
+        m_makeArguments.clear();
+    }
 
     QString makefile = workingDirectory;
 
-    if (qt4bc->subNodeBuild()) {
-        if (!qt4bc->subNodeBuild()->makefile().isEmpty())
-            makefile.append(qt4bc->subNodeBuild()->makefile());
+    if (qmakeBc->subNodeBuild()) {
+        if (!qmakeBc->subNodeBuild()->makefile().isEmpty())
+            makefile.append(qmakeBc->subNodeBuild()->makefile());
         else
             makefile.append(QLatin1String("/Makefile"));
-    } else if (!qt4bc->makefile().isEmpty()) {
+    } else if (!qmakeBc->makefile().isEmpty()) {
         makefile.append(QLatin1Char('/'));
-        makefile.append(qt4bc->makefile());
+        makefile.append(qmakeBc->makefile());
     } else {
         makefile.append(QLatin1String("/Makefile"));
     }
 
     // Check whether we need to run qmake
-    bool makefileOutDated = (qt4bc->compareToImportFrom(makefile) != QmakeBuildConfiguration::MakefileMatches);
+    bool makefileOutDated = (qmakeBc->compareToImportFrom(makefile) != QmakeBuildConfiguration::MakefileMatches);
     if (m_forced || makefileOutDated)
         m_needToRunQMake = true;
     m_forced = false;
 
     ProcessParameters *pp = processParameters();
-    pp->setMacroExpander(qt4bc->macroExpander());
+    pp->setMacroExpander(qmakeBc->macroExpander());
     pp->setWorkingDirectory(workingDirectory);
-    pp->setCommand(program.toString());
-    pp->setArguments(args);
-    pp->setEnvironment(qt4bc->environment());
-    pp->resolveAll();
+    pp->setEnvironment(qmakeBc->environment());
 
     setOutputParser(new QMakeParser);
 
-    QmakeProFileNode *node = static_cast<QmakeProject *>(qt4bc->target()->project())->rootProjectNode();
-    if (qt4bc->subNodeBuild())
-        node = qt4bc->subNodeBuild();
+    QmakeProFileNode *node = static_cast<QmakeProject *>(qmakeBc->target()->project())->rootProjectNode();
+    if (qmakeBc->subNodeBuild())
+        node = qmakeBc->subNodeBuild();
     QString proFile = node->filePath().toString();
 
     QList<ProjectExplorer::Task> tasks = qtVersion->reportIssues(proFile, workingDirectory);
@@ -249,20 +270,26 @@ bool QMakeStep::init(QList<const BuildStep *> &earlierSteps)
 
 void QMakeStep::run(QFutureInterface<bool> &fi)
 {
+    m_inputFuture = fi;
+    m_inputWatcher.setFuture(m_inputFuture.future());
+
+    fi.setProgressRange(0, static_cast<int>(State::POST_PROCESS));
+    fi.setProgressValue(0);
     if (m_scriptTemplate) {
-        fi.reportResult(true);
+        reportRunResult(fi, true);
         return;
     }
 
     if (!m_needToRunQMake) {
         emit addOutput(tr("Configuration unchanged, skipping qmake step."), BuildStep::MessageOutput);
-        fi.reportResult(true);
-        emit finished();
+        reportRunResult(fi, true);
         return;
     }
 
     m_needToRunQMake = false;
-    AbstractProcessStep::run(fi);
+
+    m_nextState = State::RUN_QMAKE;
+    runNextCommand();
 }
 
 void QMakeStep::setForced(bool b)
@@ -301,6 +328,66 @@ bool QMakeStep::processSucceeded(int exitCode, QProcess::ExitStatus status)
     return result;
 }
 
+void QMakeStep::startOneCommand(const QString &command, const QString &args)
+{
+    ProcessParameters *pp = processParameters();
+    pp->setCommand(command);
+    pp->setArguments(args);
+    pp->resolveAll();
+
+    QTC_ASSERT(!m_commandFuture || m_commandFuture->future().isFinished(), return);
+    delete m_commandFuture;
+    m_commandFuture = new QFutureInterface<bool>;
+
+    m_commandWatcher.setFuture(m_commandFuture->future());
+    AbstractProcessStep::run(*m_commandFuture);
+}
+
+void QMakeStep::runNextCommand()
+{
+    bool wasSuccess = true;
+    if (m_commandFuture) {
+        if (m_commandFuture->isCanceled())
+            wasSuccess = false;
+        else if (m_commandFuture->isFinished())
+            wasSuccess = m_commandFuture->future().result();
+        else
+            wasSuccess = false; // should not happen
+    }
+
+    delete m_commandFuture;
+    m_commandFuture = nullptr;
+
+    if (!wasSuccess)
+        m_nextState = State::POST_PROCESS;
+
+    m_inputFuture.setProgressValue(static_cast<int>(m_nextState));
+
+    switch (m_nextState) {
+    case State::IDLE:
+        return;
+    case State::RUN_QMAKE:
+        setOutputParser(new QMakeParser);
+        m_nextState = (m_runMakeQmake ? State::RUN_MAKE_QMAKE_ALL : State::POST_PROCESS);
+        startOneCommand(m_qmakeExecutable, m_qmakeArguments);
+        return;
+    case State::RUN_MAKE_QMAKE_ALL:
+        {
+            GnuMakeParser *parser = new GnuMakeParser;
+            parser->setWorkingDirectory(processParameters()->workingDirectory());
+            setOutputParser(parser);
+            m_nextState = State::POST_PROCESS;
+            startOneCommand(m_makeExecutable, m_makeArguments);
+        }
+        return;
+    case State::POST_PROCESS:
+        m_nextState = State::IDLE;
+        reportRunResult(m_inputFuture, wasSuccess);
+        m_inputFuture = QFutureInterface<bool>();
+        return;
+    }
+}
+
 void QMakeStep::setUserArguments(const QString &arguments)
 {
     if (m_userArgs == arguments)
@@ -366,12 +453,54 @@ void QMakeStep::setSeparateDebugInfo(bool enable)
     qmakeBuildConfiguration()->emitProFileEvaluateNeeded();
 }
 
+QString QMakeStep::makeCommand() const
+{
+    MakeStep *ms = qobject_cast<BuildStepList *>(parent())->firstOfType<MakeStep>();
+    return ms ? ms->effectiveMakeCommand() : QString();
+}
+
+QString QMakeStep::makeArguments() const
+{
+    QString args;
+    if (QmakeBuildConfiguration *qmakeBc = qmakeBuildConfiguration()) {
+        const QString makefile = qmakeBc->makefile();
+        if (!makefile.isEmpty()) {
+            Utils::QtcProcess::addArg(&args, "-f");
+            Utils::QtcProcess::addArg(&args, makefile);
+        }
+    }
+    Utils::QtcProcess::addArg(&args, "qmake_all");
+    return args;
+}
+
+QString QMakeStep::effectiveQMakeCall() const
+{
+    BaseQtVersion *qtVersion = QtKitInformation::qtVersion(target()->kit());
+    QString qmake = qtVersion ? qtVersion->qmakeCommand().fileName() : QString();
+    if (qmake.isEmpty())
+        qmake = tr("<no Qt version>");
+    QString make = makeCommand();
+    if (make.isEmpty())
+        make = tr("<no Make step found>");
+
+    QString result = qmake;
+    if (qtVersion) {
+        result += QLatin1Char(' ') + allArguments(qtVersion);
+        if (qtVersion->qtVersion() >= QtVersionNumber(5, 0, 0))
+            result.append(QString::fromLatin1(" && %1 %2").arg(make).arg(makeArguments()));
+    }
+    return result;
+}
+
 QStringList QMakeStep::parserArguments()
 {
     QStringList result;
-    for (QtcProcess::ConstArgIterator ait(allArguments()); ait.next(); )
+    BaseQtVersion *qt = QtKitInformation::qtVersion(target()->kit());
+    QTC_ASSERT(qt, return QStringList());
+    for (QtcProcess::ConstArgIterator ait(allArguments(qt)); ait.next(); ) {
         if (ait.isSimple())
             result << ait.value();
+    }
     return result;
 }
 
@@ -380,7 +509,7 @@ QString QMakeStep::userArguments()
     return m_userArgs;
 }
 
-FileName QMakeStep::mkspec()
+FileName QMakeStep::mkspec() const
 {
     QString additionalArguments = m_userArgs;
     for (QtcProcess::ArgIterator ait(&additionalArguments); ait.next(); ) {
@@ -448,10 +577,11 @@ QMakeStepConfigWidget::QMakeStepConfigWidget(QMakeStep *step)
     updateQmlDebuggingOption();
     updateQtQuickCompilerOption();
 
-    connect(m_ui->qmakeAdditonalArgumentsLineEdit, SIGNAL(textEdited(QString)),
-            this, SLOT(qmakeArgumentsLineEdited()));
-    connect(m_ui->buildConfigurationComboBox, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(buildConfigurationSelected()));
+    connect(m_ui->qmakeAdditonalArgumentsLineEdit, &QLineEdit::textEdited,
+            this, &QMakeStepConfigWidget::qmakeArgumentsLineEdited);
+    connect(m_ui->buildConfigurationComboBox,
+            static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+            this, &QMakeStepConfigWidget::buildConfigurationSelected);
     connect(m_ui->qmlDebuggingLibraryCheckBox, &QCheckBox::toggled,
             this, &QMakeStepConfigWidget::linkQmlDebuggingLibraryChecked);
     connect(m_ui->qmlDebuggingLibraryCheckBox, &QCheckBox::clicked,
@@ -464,21 +594,21 @@ QMakeStepConfigWidget::QMakeStepConfigWidget(QMakeStep *step)
             this, &QMakeStepConfigWidget::separateDebugInfoChecked);
     connect(m_ui->separateDebugInfoCheckBox, &QCheckBox::clicked,
             this, &QMakeStepConfigWidget::askForRebuild);
-    connect(step, SIGNAL(userArgumentsChanged()),
-            this, SLOT(userArgumentsChanged()));
-    connect(step, SIGNAL(linkQmlDebuggingLibraryChanged()),
-            this, SLOT(linkQmlDebuggingLibraryChanged()));
+    connect(step, &QMakeStep::userArgumentsChanged,
+            this, &QMakeStepConfigWidget::userArgumentsChanged);
+    connect(step, &QMakeStep::linkQmlDebuggingLibraryChanged,
+            this, &QMakeStepConfigWidget::linkQmlDebuggingLibraryChanged);
     connect(step->project(), &Project::projectLanguagesUpdated,
             this, &QMakeStepConfigWidget::linkQmlDebuggingLibraryChanged);
     connect(step, &QMakeStep::useQtQuickCompilerChanged,
             this, &QMakeStepConfigWidget::useQtQuickCompilerChanged);
     connect(step, &QMakeStep::separateDebugInfoChanged,
             this, &QMakeStepConfigWidget::separateDebugInfoChanged);
-    connect(step->qmakeBuildConfiguration(), SIGNAL(qmakeBuildConfigurationChanged()),
-            this, SLOT(qmakeBuildConfigChanged()));
-    connect(step->target(), SIGNAL(kitChanged()), this, SLOT(qtVersionChanged()));
-    connect(QtSupport::QtVersionManager::instance(), SIGNAL(dumpUpdatedFor(Utils::FileName)),
-            this, SLOT(qtVersionChanged()));
+    connect(step->qmakeBuildConfiguration(), &QmakeBuildConfiguration::qmakeBuildConfigurationChanged,
+            this, &QMakeStepConfigWidget::qmakeBuildConfigChanged);
+    connect(step->target(), &Target::kitChanged, this, &QMakeStepConfigWidget::qtVersionChanged);
+    connect(QtVersionManager::instance(), &QtVersionManager::dumpUpdatedFor,
+            this, &QMakeStepConfigWidget::qtVersionChanged);
 }
 
 QMakeStepConfigWidget::~QMakeStepConfigWidget()
@@ -512,7 +642,7 @@ void QMakeStepConfigWidget::qtVersionChanged()
 void QMakeStepConfigWidget::qmakeBuildConfigChanged()
 {
     QmakeBuildConfiguration *bc = m_step->qmakeBuildConfiguration();
-    bool debug = bc->qmakeBuildConfiguration() & QtSupport::BaseQtVersion::DebugBuild;
+    bool debug = bc->qmakeBuildConfiguration() & BaseQtVersion::DebugBuild;
     m_ignoreChange = true;
     m_ui->buildConfigurationComboBox->setCurrentIndex(debug? 0 : 1);
     m_ignoreChange = false;
@@ -575,11 +705,11 @@ void QMakeStepConfigWidget::buildConfigurationSelected()
     if (m_ignoreChange)
         return;
     QmakeBuildConfiguration *bc = m_step->qmakeBuildConfiguration();
-    QtSupport::BaseQtVersion::QmakeBuildConfigs buildConfiguration = bc->qmakeBuildConfiguration();
+    BaseQtVersion::QmakeBuildConfigs buildConfiguration = bc->qmakeBuildConfiguration();
     if (m_ui->buildConfigurationComboBox->currentIndex() == 0) { // debug
-        buildConfiguration = buildConfiguration | QtSupport::BaseQtVersion::DebugBuild;
+        buildConfiguration = buildConfiguration | BaseQtVersion::DebugBuild;
     } else {
-        buildConfiguration = buildConfiguration & ~QtSupport::BaseQtVersion::DebugBuild;
+        buildConfiguration = buildConfiguration & ~BaseQtVersion::DebugBuild;
     }
     m_ignoreChange = true;
     bc->setQMakeBuildConfiguration(buildConfiguration);
@@ -610,7 +740,7 @@ void QMakeStepConfigWidget::askForRebuild()
     question->setText(tr("The option will only take effect if the project is recompiled. Do you want to recompile now?"));
     question->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
     question->setModal(true);
-    connect(question, SIGNAL(finished(int)), this, SLOT(recompileMessageBoxFinished(int)));
+    connect(question, &QDialog::finished, this, &QMakeStepConfigWidget::recompileMessageBoxFinished);
     question->show();
 }
 
@@ -644,13 +774,13 @@ void QMakeStepConfigWidget::separateDebugInfoChecked(bool checked)
 
 void QMakeStepConfigWidget::updateSummaryLabel()
 {
-    QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(m_step->target()->kit());
+    BaseQtVersion *qtVersion = QtKitInformation::qtVersion(m_step->target()->kit());
     if (!qtVersion) {
         setSummaryText(tr("<b>qmake:</b> No Qt version set. Cannot run qmake."));
         return;
     }
     // We don't want the full path to the .pro file
-    QString args = m_step->allArguments(true);
+    QString args = m_step->allArguments(qtVersion, true);
     // And we only use the .pro filename not the full path
     QString program = qtVersion->qmakeCommand().fileName();
     setSummaryText(tr("<b>qmake:</b> %1 %2").arg(program, args));
@@ -659,7 +789,7 @@ void QMakeStepConfigWidget::updateSummaryLabel()
 void QMakeStepConfigWidget::updateQmlDebuggingOption()
 {
     QString warningText;
-    bool supported = QtSupport::BaseQtVersion::isQmlDebuggingSupported(m_step->target()->kit(),
+    bool supported = BaseQtVersion::isQmlDebuggingSupported(m_step->target()->kit(),
                                                                        &warningText);
 
     m_ui->qmlDebuggingLibraryCheckBox->setEnabled(supported);
@@ -677,7 +807,7 @@ void QMakeStepConfigWidget::updateQmlDebuggingOption()
 void QMakeStepConfigWidget::updateQtQuickCompilerOption()
 {
     QString warningText;
-    bool supported = QtSupport::BaseQtVersion::isQtQuickCompilerSupported(m_step->target()->kit(),
+    bool supported = BaseQtVersion::isQtQuickCompilerSupported(m_step->target()->kit(),
                                                                           &warningText);
     m_ui->qtQuickCompilerCheckBox->setEnabled(supported);
     m_ui->qtQuickCompilerLabel->setText(tr("Enable Qt Quick Compiler:"));
@@ -691,11 +821,7 @@ void QMakeStepConfigWidget::updateQtQuickCompilerOption()
 
 void QMakeStepConfigWidget::updateEffectiveQMakeCall()
 {
-    QtSupport::BaseQtVersion *qtVersion = QtSupport::QtKitInformation::qtVersion(m_step->target()->kit());
-    QString program = tr("<No Qt version>");
-    if (qtVersion)
-        program = qtVersion->qmakeCommand().fileName();
-    m_ui->qmakeArgumentsEdit->setPlainText(program + QLatin1Char(' ') + m_step->allArguments());
+    m_ui->qmakeArgumentsEdit->setPlainText(m_step->effectiveQMakeCall());
 }
 
 void QMakeStepConfigWidget::recompileMessageBoxFinished(int button)
@@ -731,67 +857,27 @@ QMakeStepFactory::QMakeStepFactory(QObject *parent) :
 {
 }
 
-bool QMakeStepFactory::canCreate(BuildStepList *parent, Core::Id id) const
+QList<BuildStepInfo> QMakeStepFactory::availableSteps(BuildStepList *parent) const
 {
     if (parent->id() != ProjectExplorer::Constants::BUILDSTEPS_BUILD)
-        return false;
+        return {};
     if (!qobject_cast<QmakeBuildConfiguration *>(parent->parent()))
-        return false;
-    return id == QMAKE_BS_ID;
+        return {};
+
+    return {{ QMAKE_BS_ID, tr("qmake"), BuildStepInfo::UniqueStep }};
 }
 
 ProjectExplorer::BuildStep *QMakeStepFactory::create(BuildStepList *parent, Core::Id id)
 {
-    if (!canCreate(parent, id))
-        return 0;
+    Q_UNUSED(id)
     return new QMakeStep(parent);
 }
 
-bool QMakeStepFactory::canClone(BuildStepList *parent, BuildStep *source) const
-{
-    return canCreate(parent, source->id());
-}
-
 ProjectExplorer::BuildStep *QMakeStepFactory::clone(BuildStepList *parent, ProjectExplorer::BuildStep *source)
 {
-    if (!canClone(parent, source))
-        return 0;
     return new QMakeStep(parent, qobject_cast<QMakeStep *>(source));
 }
 
-bool QMakeStepFactory::canRestore(BuildStepList *parent, const QVariantMap &map) const
-{
-    return canCreate(parent, ProjectExplorer::idFromMap(map));
-}
-
-ProjectExplorer::BuildStep *QMakeStepFactory::restore(BuildStepList *parent, const QVariantMap &map)
-{
-    if (!canRestore(parent, map))
-        return 0;
-    QMakeStep *bs = new QMakeStep(parent);
-    if (bs->fromMap(map))
-        return bs;
-    delete bs;
-    return 0;
-}
-
-QList<Core::Id> QMakeStepFactory::availableCreationIds(ProjectExplorer::BuildStepList *parent) const
-{
-    if (parent->id() == ProjectExplorer::Constants::BUILDSTEPS_BUILD)
-        if (QmakeBuildConfiguration *bc = qobject_cast<QmakeBuildConfiguration *>(parent->parent()))
-            if (!bc->qmakeStep())
-                return QList<Core::Id>() << Core::Id(QMAKE_BS_ID);
-    return QList<Core::Id>();
-}
-
-QString QMakeStepFactory::displayNameForId(Core::Id id) const
-{
-    if (id == QMAKE_BS_ID)
-        return tr("qmake");
-    return QString();
-}
-
-
 QMakeStepConfig::TargetArchConfig QMakeStepConfig::targetArchFor(const Abi &targetAbi, const BaseQtVersion *version)
 {
     QMakeStepConfig::TargetArchConfig arch = QMakeStepConfig::NoArch;
@@ -806,9 +892,9 @@ QMakeStepConfig::TargetArchConfig QMakeStepConfig::targetArchFor(const Abi &targ
                 arch = QMakeStepConfig::X86_64;
         } else if (targetAbi.architecture() == ProjectExplorer::Abi::PowerPCArchitecture) {
             if (targetAbi.wordWidth() == 32)
-                arch = QMakeStepConfig::PPC;
+                arch = QMakeStepConfig::PowerPC;
             else if (targetAbi.wordWidth() == 64)
-                arch = QMakeStepConfig::PPC64;
+                arch = QMakeStepConfig::PowerPC64;
         }
     }
     return arch;
@@ -838,9 +924,9 @@ QStringList QMakeStepConfig::toArguments() const
         arguments << QLatin1String("CONFIG+=x86");
     else if (archConfig == X86_64)
         arguments << QLatin1String("CONFIG+=x86_64");
-    else if (archConfig == PPC)
+    else if (archConfig == PowerPC)
         arguments << QLatin1String("CONFIG+=ppc");
-    else if (archConfig == PPC64)
+    else if (archConfig == PowerPC64)
         arguments << QLatin1String("CONFIG+=ppc64");
 
     // TODO: make that depend on the actual Qt version that is used
@@ -861,4 +947,3 @@ QStringList QMakeStepConfig::toArguments() const
 
     return arguments;
 }
-
