@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -33,12 +28,15 @@
 #include "propertyeditorvalue.h"
 #include "propertyeditortransaction.h"
 #include <qmldesignerconstants.h>
+#include <qmldesignerplugin.h>
 
 #include <qmlobjectnode.h>
 #include <nodemetainfo.h>
 #include <variantproperty.h>
 #include <bindingproperty.h>
 
+#include <theming.h>
+
 #include <coreplugin/icore.h>
 #include <qmljs/qmljssimplereader.h>
 #include <utils/algorithm.h>
@@ -93,18 +91,21 @@ PropertyEditorQmlBackend::PropertyEditorQmlBackend(PropertyEditorView *propertyE
         m_view(new Quick2PropertyEditorView), m_propertyEditorTransaction(new PropertyEditorTransaction(propertyEditor)), m_dummyPropertyEditorValue(new PropertyEditorValue()),
         m_contextObject(new PropertyEditorContextObject())
 {
-    Q_ASSERT(QFileInfo(":/images/button_normal.png").exists());
+    Q_ASSERT(QFileInfo(QLatin1String(":/images/button_normal.png")).exists());
 
-    m_view->engine()->setOutputWarningsToStandardError(
-                !qgetenv("QTCREATOR_QTQUICKDESIGNER_PROPERTYEDITOR_SHOW_WARNINGS").isEmpty());
+    m_view->engine()->setOutputWarningsToStandardError(QmlDesignerPlugin::instance()
+        ->settings().value(DesignerSettingsKey::SHOW_PROPERTYEDITOR_WARNINGS).toBool());
 
     m_view->engine()->addImportPath(propertyEditorResourcesPath());
-    m_dummyPropertyEditorValue->setValue("#000000");
-    context()->setContextProperty("dummyBackendValue", m_dummyPropertyEditorValue.data());
+    m_dummyPropertyEditorValue->setValue(QLatin1String("#000000"));
+    context()->setContextProperty(QLatin1String("dummyBackendValue"), m_dummyPropertyEditorValue.data());
     m_contextObject->setBackendValues(&m_backendValuesPropertyMap);
     m_contextObject->insertInQmlContext(context());
 
-    QObject::connect(&m_backendValuesPropertyMap, SIGNAL(valueChanged(QString,QVariant)), propertyEditor, SLOT(changeValue(QString)));
+    Theming::insertTheme(&m_themeProperties);
+    context()->setContextProperty(QLatin1String("creatorTheme"), &m_themeProperties);
+
+    QObject::connect(&m_backendValuesPropertyMap, &DesignerPropertyMap::valueChanged, propertyEditor, &PropertyEditorView::changeValue);
 }
 
 PropertyEditorQmlBackend::~PropertyEditorQmlBackend()
@@ -115,21 +116,68 @@ void PropertyEditorQmlBackend::setupPropertyEditorValue(const PropertyName &name
 {
     QmlDesigner::PropertyName propertyName(name);
     propertyName.replace('.', '_');
-    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(backendValuesPropertyMap().value(propertyName)));
+    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(backendValuesPropertyMap().value(QString::fromUtf8(propertyName))));
     if (!valueObject) {
         valueObject = new PropertyEditorValue(&backendValuesPropertyMap());
-        QObject::connect(valueObject, SIGNAL(valueChanged(QString,QVariant)), &backendValuesPropertyMap(), SIGNAL(valueChanged(QString,QVariant)));
-        QObject::connect(valueObject, SIGNAL(expressionChanged(QString)), propertyEditor, SLOT(changeExpression(QString)));
+        QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
+        QObject::connect(valueObject, &PropertyEditorValue::expressionChanged, propertyEditor, &PropertyEditorView::changeExpression);
         backendValuesPropertyMap().insert(QString::fromUtf8(propertyName), QVariant::fromValue(valueObject));
     }
     valueObject->setName(propertyName);
-    if (type == "QColor")
-        valueObject->setValue(QVariant("#000000"));
+    if (type == QLatin1String("QColor"))
+        valueObject->setValue(QVariant(QLatin1String("#000000")));
     else
         valueObject->setValue(QVariant(1));
 
 }
 
+static PropertyNameList layoutAttachedPropertiesNames()
+{
+    PropertyNameList propertyNames;
+    propertyNames << "alignment" << "column" << "columnSpan" << "fillHeight" << "fillWidth"
+                  << "maximumHeight" << "maximumWidth" << "minimumHeight" << "minimumWidth"
+                  << "preferredHeight" << "preferredWidth" << "row"<< "rowSpan";
+
+    return propertyNames;
+}
+
+QVariant properDefaultLayoutAttachedProperties(const QmlObjectNode &qmlObjectNode, const PropertyName &propertyName)
+{
+    QVariant value = qmlObjectNode.modelValue(propertyName);
+
+    if (value.isValid())
+        return value;
+
+    if ("fillHeight" == propertyName || "fillWidth" == propertyName)
+        return false;
+
+      if ("minimumWidth" == propertyName || "minimumHeight" == propertyName)
+          return 0;
+
+      if ("preferredWidth" == propertyName || "preferredHeight" == propertyName)
+          return -1;
+
+      if ("maximumWidth" == propertyName || "maximumHeight" == propertyName)
+          return 0xffff;
+
+     if ("columnSpan" == propertyName || "rowSpan" == propertyName)
+         return 1;
+
+    return QVariant();
+}
+
+void PropertyEditorQmlBackend::setupLayoutAttachedProperties(const QmlObjectNode &qmlObjectNode, PropertyEditorView *propertyEditor)
+{
+    if (QmlItemNode(qmlObjectNode).isInLayout()) {
+
+        static const PropertyNameList propertyNames = layoutAttachedPropertiesNames();
+
+        foreach (const PropertyName &propertyName, propertyNames) {
+            createPropertyEditorValue(qmlObjectNode, "Layout." + propertyName, properDefaultLayoutAttachedProperties(qmlObjectNode, propertyName), propertyEditor);
+        }
+    }
+}
+
 void PropertyEditorQmlBackend::createPropertyEditorValue(const QmlObjectNode &qmlObjectNode,
                                              const PropertyName &name,
                                              const QVariant &value,
@@ -137,11 +185,11 @@ void PropertyEditorQmlBackend::createPropertyEditorValue(const QmlObjectNode &qm
 {
     PropertyName propertyName(name);
     propertyName.replace('.', '_');
-    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(backendValuesPropertyMap().value(propertyName)));
+    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(backendValuesPropertyMap().value(QString::fromUtf8(propertyName))));
     if (!valueObject) {
         valueObject = new PropertyEditorValue(&backendValuesPropertyMap());
-        QObject::connect(valueObject, SIGNAL(valueChanged(QString,QVariant)), &backendValuesPropertyMap(), SIGNAL(valueChanged(QString,QVariant)));
-        QObject::connect(valueObject, SIGNAL(expressionChanged(QString)), propertyEditor, SLOT(changeExpression(QString)));
+        QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
+        QObject::connect(valueObject, &PropertyEditorValue::expressionChanged, propertyEditor, &PropertyEditorView::changeExpression);
         backendValuesPropertyMap().insert(QString::fromUtf8(propertyName), QVariant::fromValue(valueObject));
     }
     valueObject->setName(name);
@@ -158,7 +206,7 @@ void PropertyEditorQmlBackend::createPropertyEditorValue(const QmlObjectNode &qm
         qmlObjectNode.modelNode().property(propertyName).isBindingProperty()) {
         valueObject->setExpression(qmlObjectNode.modelNode().bindingProperty(propertyName).expression());
     } else {
-        if (qmlObjectNode.hasBindingProperty(propertyName))
+        if (qmlObjectNode.hasBindingProperty(name))
             valueObject->setExpression(qmlObjectNode.expression(name));
         else
             valueObject->setExpression(qmlObjectNode.instanceValue(name).toString());
@@ -169,7 +217,7 @@ void PropertyEditorQmlBackend::setValue(const QmlObjectNode & qmlObjectNode, con
 {
     PropertyName propertyName = name;
     propertyName.replace('.', '_');
-    PropertyEditorValue *propertyValue = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(propertyName)));
+    PropertyEditorValue *propertyValue = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QString::fromUtf8(propertyName))));
     if (propertyValue) {
         propertyValue->setValue(value);
 
@@ -180,7 +228,6 @@ void PropertyEditorQmlBackend::setValue(const QmlObjectNode & qmlObjectNode, con
     }
 }
 
-
 QQmlContext *PropertyEditorQmlBackend::context() {
     return m_view->rootContext();
 }
@@ -223,45 +270,45 @@ void PropertyEditorQmlBackend::setup(const QmlObjectNode &qmlObjectNode, const Q
         foreach (const PropertyName &propertyName, qmlObjectNode.modelNode().metaInfo().propertyNames())
             createPropertyEditorValue(qmlObjectNode, propertyName, qmlObjectNode.instanceValue(propertyName), propertyEditor);
 
+        setupLayoutAttachedProperties(qmlObjectNode, propertyEditor);
+
         // className
-        PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value("className")));
+        PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QLatin1String("className"))));
         if (!valueObject)
             valueObject = new PropertyEditorValue(&m_backendValuesPropertyMap);
         valueObject->setName("className");
         valueObject->setModelNode(qmlObjectNode.modelNode());
         valueObject->setValue(qmlObjectNode.modelNode().simplifiedTypeName());
-        QObject::connect(valueObject, SIGNAL(valueChanged(QString,QVariant)), &m_backendValuesPropertyMap, SIGNAL(valueChanged(QString,QVariant)));
-        m_backendValuesPropertyMap.insert("className", QVariant::fromValue(valueObject));
+        QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
+        m_backendValuesPropertyMap.insert(QLatin1String("className"), QVariant::fromValue(valueObject));
 
         // id
-        valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value("id")));
+        valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QLatin1String("id"))));
         if (!valueObject)
             valueObject = new PropertyEditorValue(&m_backendValuesPropertyMap);
         valueObject->setName("id");
         valueObject->setValue(qmlObjectNode.id());
-        QObject::connect(valueObject, SIGNAL(valueChanged(QString,QVariant)), &m_backendValuesPropertyMap, SIGNAL(valueChanged(QString,QVariant)));
-        m_backendValuesPropertyMap.insert("id", QVariant::fromValue(valueObject));
+        QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
+        m_backendValuesPropertyMap.insert(QLatin1String("id"), QVariant::fromValue(valueObject));
 
         QmlItemNode itemNode(qmlObjectNode.modelNode());
 
         // anchors
         m_backendAnchorBinding.setup(qmlObjectNode.modelNode());
-        context()->setContextProperty("anchorBackend", &m_backendAnchorBinding);
-
-
-        context()->setContextProperty("transaction", m_propertyEditorTransaction.data());
+        context()->setContextProperty(QLatin1String("anchorBackend"), &m_backendAnchorBinding);
 
+        context()->setContextProperty(QLatin1String("transaction"), m_propertyEditorTransaction.data());
 
         // model node
         m_backendModelNode.setup(qmlObjectNode.modelNode());
-        context()->setContextProperty("modelNodeBackend", &m_backendModelNode);
+        context()->setContextProperty(QLatin1String("modelNodeBackend"), &m_backendModelNode);
 
         contextObject()->setSpecificsUrl(qmlSpecificsFile);
 
         contextObject()->setStateName(stateName);
         if (!qmlObjectNode.isValid())
             return;
-        context()->setContextProperty("propertyCount", QVariant(qmlObjectNode.modelNode().properties().count()));
+        context()->setContextProperty(QLatin1String("propertyCount"), QVariant(qmlObjectNode.modelNode().properties().count()));
 
         contextObject()->setIsBaseState(qmlObjectNode.isInBaseState());
         contextObject()->setSelectionChanged(false);
@@ -287,32 +334,32 @@ void PropertyEditorQmlBackend::setup(const QmlObjectNode &qmlObjectNode, const Q
 
 void PropertyEditorQmlBackend::initialSetup(const TypeName &typeName, const QUrl &qmlSpecificsFile, PropertyEditorView *propertyEditor)
 {
-    NodeMetaInfo metaInfo = propertyEditor->model()->metaInfo(typeName, 4, 7);
+    NodeMetaInfo metaInfo = propertyEditor->model()->metaInfo(typeName);
 
     foreach (const PropertyName &propertyName, metaInfo.propertyNames())
-        setupPropertyEditorValue(propertyName, propertyEditor, metaInfo.propertyTypeName(propertyName));
+        setupPropertyEditorValue(propertyName, propertyEditor, QString::fromUtf8(metaInfo.propertyTypeName(propertyName)));
 
-    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value("className")));
+    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QLatin1String("className"))));
     if (!valueObject)
         valueObject = new PropertyEditorValue(&m_backendValuesPropertyMap);
     valueObject->setName("className");
 
     valueObject->setValue(typeName);
-    QObject::connect(valueObject, SIGNAL(valueChanged(QString,QVariant)), &m_backendValuesPropertyMap, SIGNAL(valueChanged(QString,QVariant)));
-    m_backendValuesPropertyMap.insert("className", QVariant::fromValue(valueObject));
+    QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
+    m_backendValuesPropertyMap.insert(QLatin1String("className"), QVariant::fromValue(valueObject));
 
     // id
-    valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value("id")));
+    valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QLatin1String("id"))));
     if (!valueObject)
         valueObject = new PropertyEditorValue(&m_backendValuesPropertyMap);
     valueObject->setName("id");
     valueObject->setValue("id");
-    QObject::connect(valueObject, SIGNAL(valueChanged(QString,QVariant)), &m_backendValuesPropertyMap, SIGNAL(valueChanged(QString,QVariant)));
-    m_backendValuesPropertyMap.insert("id", QVariant::fromValue(valueObject));
+    QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
+    m_backendValuesPropertyMap.insert(QLatin1String("id"), QVariant::fromValue(valueObject));
 
-    context()->setContextProperty("anchorBackend", &m_backendAnchorBinding);
-    context()->setContextProperty("modelNodeBackend", &m_backendModelNode);
-    context()->setContextProperty("transaction", m_propertyEditorTransaction.data());
+    context()->setContextProperty(QLatin1String("anchorBackend"), &m_backendAnchorBinding);
+    context()->setContextProperty(QLatin1String("modelNodeBackend"), &m_backendModelNode);
+    context()->setContextProperty(QLatin1String("transaction"), m_propertyEditorTransaction.data());
 
     contextObject()->setSpecificsUrl(qmlSpecificsFile);
 
@@ -338,9 +385,9 @@ QString PropertyEditorQmlBackend::templateGeneration(NodeMetaInfo type,
 
     QStringList imports = variantToStringList(templateConfiguration()->property(QStringLiteral("imports")));
 
-    QString qmlTemplate = imports.join(QStringLiteral("\n")) + QLatin1Char('\n');
+    QString qmlTemplate = imports.join(QLatin1Char('\n')) + QLatin1Char('\n');
     qmlTemplate += QStringLiteral("Section {\n");
-    qmlTemplate += QStringLiteral("caption: \"%1\"\n").arg(QString::fromUtf8(objectNode.modelNode().simplifiedTypeName()));
+    qmlTemplate += QStringLiteral("caption: \"%1\"\n").arg(objectNode.modelNode().simplifiedTypeName());
     qmlTemplate += QStringLiteral("SectionLayout {\n");
 
     QList<PropertyName> orderedList = type.propertyNames();
@@ -356,18 +403,18 @@ QString PropertyEditorQmlBackend::templateGeneration(NodeMetaInfo type,
 
         properName.replace('.', '_');
 
-        QString typeName = type.propertyTypeName(name);
+        TypeName typeName = type.propertyTypeName(name);
         //alias resolution only possible with instance
-        if (typeName == QStringLiteral("alias") && objectNode.isValid())
+        if (typeName == "alias" && objectNode.isValid())
             typeName = objectNode.instanceType(name);
 
         if (!superType.hasProperty(name) && type.propertyIsWritable(name) && !name.contains(".")) {
             foreach (const QmlJS::SimpleReaderNode::Ptr &node, templateConfiguration()->children())
-                if (variantToStringList(node->property(QStringLiteral("typeNames"))).contains(typeName)) {
+                if (variantToStringList(node->property(QStringLiteral("typeNames"))).contains(QString::fromLatin1(typeName))) {
                     const QString fileName = propertyTemplatesPath() + node->property(QStringLiteral("sourceFile")).toString();
                     QFile file(fileName);
                     if (file.open(QIODevice::ReadOnly)) {
-                        QString source = file.readAll();
+                        QString source = QString::fromUtf8(file.readAll());
                         file.close();
                         qmlTemplate += source.arg(QString::fromUtf8(name)).arg(QString::fromUtf8(properName));
                         emptyTemplate = false;
@@ -386,24 +433,22 @@ QString PropertyEditorQmlBackend::templateGeneration(NodeMetaInfo type,
     return qmlTemplate;
 }
 
-QUrl PropertyEditorQmlBackend::getQmlFileUrl(const QString &relativeTypeName, const NodeMetaInfo &info)
+QUrl PropertyEditorQmlBackend::getQmlFileUrl(const TypeName &relativeTypeName, const NodeMetaInfo &info)
 {
-    return fileToUrl(locateQmlFile(info, fixTypeNameForPanes(relativeTypeName) + QStringLiteral(".qml")));
+    return fileToUrl(locateQmlFile(info, QString::fromUtf8(fixTypeNameForPanes(relativeTypeName) + ".qml")));
 }
 
-QString PropertyEditorQmlBackend::fixTypeNameForPanes(const QString &typeName)
+TypeName PropertyEditorQmlBackend::fixTypeNameForPanes(const TypeName &typeName)
 {
-    QString fixedTypeName = typeName;
+    TypeName fixedTypeName = typeName;
     fixedTypeName.replace('.', '/');
     return fixedTypeName;
 }
 
-QString PropertyEditorQmlBackend::qmlFileName(const NodeMetaInfo &nodeInfo)
+TypeName PropertyEditorQmlBackend::qmlFileName(const NodeMetaInfo &nodeInfo)
 {
-    if (nodeInfo.typeName().split('.').last() == "QDeclarativeItem")
-        return "QtQuick/ItemPane.qml";
-    const QString fixedTypeName = fixTypeNameForPanes(nodeInfo.typeName());
-    return fixedTypeName + QStringLiteral("Pane.qml");
+    const TypeName fixedTypeName = fixTypeNameForPanes(nodeInfo.typeName());
+    return fixedTypeName + "Pane.qml";
 }
 
 QUrl PropertyEditorQmlBackend::fileToUrl(const QString &filePath)  {
@@ -412,8 +457,8 @@ QUrl PropertyEditorQmlBackend::fileToUrl(const QString &filePath)  {
     if (filePath.isEmpty())
         return fileUrl;
 
-    if (filePath.startsWith(QLatin1Char(':'))) {
-        fileUrl.setScheme("qrc");
+    if (filePath.startsWith(':')) {
+        fileUrl.setScheme(QLatin1String("qrc"));
         QString path = filePath;
         path.remove(0, 1); // remove trailing ':'
         fileUrl.setPath(path);
@@ -436,7 +481,7 @@ QString PropertyEditorQmlBackend::fileFromUrl(const QUrl &url)
 
 bool PropertyEditorQmlBackend::checkIfUrlExists(const QUrl &url)
 {
-    return (QFileInfo(fileFromUrl(url)).exists());
+    return QFileInfo::exists(fileFromUrl(url));
 }
 
 void PropertyEditorQmlBackend::emitSelectionToBeChanged()
@@ -449,6 +494,13 @@ void PropertyEditorQmlBackend::emitSelectionChanged()
     m_backendModelNode.emitSelectionChanged();
 }
 
+void PropertyEditorQmlBackend::setValueforLayoutAttachedProperties(const QmlObjectNode &qmlObjectNode, const PropertyName &name)
+{
+    PropertyName propertyName = name;
+    propertyName.replace("Layout.", "");
+    setValue(qmlObjectNode,  name, properDefaultLayoutAttachedProperties(qmlObjectNode, propertyName));
+}
+
 QUrl PropertyEditorQmlBackend::getQmlUrlForModelNode(const ModelNode &modelNode, TypeName &className)
 {
     if (modelNode.isValid()) {
@@ -457,14 +509,14 @@ QUrl PropertyEditorQmlBackend::getQmlUrlForModelNode(const ModelNode &modelNode,
         hierarchy.append(modelNode.metaInfo().superClasses());
 
         foreach (const NodeMetaInfo &info, hierarchy) {
-            QUrl fileUrl = fileToUrl(locateQmlFile(info, qmlFileName(info)));
+            QUrl fileUrl = fileToUrl(locateQmlFile(info, QString::fromUtf8(qmlFileName(info))));
             if (fileUrl.isValid()) {
                 className = info.typeName();
                 return fileUrl;
             }
         }
     }
-    return fileToUrl(QDir(propertyEditorResourcesPath()).filePath("QtQuick/emptyPane.qml"));
+    return fileToUrl(QDir(propertyEditorResourcesPath()).filePath(QLatin1String("QtQuick/emptyPane.qml")));
 }
 
 QString PropertyEditorQmlBackend::locateQmlFile(const NodeMetaInfo &info, const QString &relativePath)
@@ -473,6 +525,7 @@ QString PropertyEditorQmlBackend::locateQmlFile(const NodeMetaInfo &info, const
 
     static QDir resourcesDir(QStringLiteral(":/propertyEditorQmlSources"));
     QDir importDir(info.importDirectoryPath() + QLatin1String(Constants::QML_DESIGNER_SUBFOLDER));
+    QDir importDirVersion(info.importDirectoryPath() + QStringLiteral(".") + QString::number(info.majorVersion()) + QLatin1String(Constants::QML_DESIGNER_SUBFOLDER));
 
     const QString versionString = QStringLiteral("_") + QString::number(info.majorVersion())
             + QStringLiteral("_")
@@ -484,6 +537,14 @@ QString PropertyEditorQmlBackend::locateQmlFile(const NodeMetaInfo &info, const
 
     //Check for qml files with versions first
     const QString withoutDirWithVersion = relativePathWithVersion.split(QStringLiteral("/")).last();
+
+    const QString withoutDir = relativePath.split(QStringLiteral("/")).last();
+
+    if (importDirVersion.exists(withoutDir))
+        return importDirVersion.absoluteFilePath(withoutDir);
+
+
+
     if (importDir.exists(relativePathWithVersion))
         return importDir.absoluteFilePath(relativePathWithVersion);
     if (importDir.exists(withoutDirWithVersion)) //Since we are in a subfolder of the import we do not require the directory
@@ -493,7 +554,7 @@ QString PropertyEditorQmlBackend::locateQmlFile(const NodeMetaInfo &info, const
     if (resourcesDir.exists(relativePathWithVersion))
         return resourcesDir.absoluteFilePath(relativePathWithVersion);
 
-    const QString withoutDir = relativePath.split(QStringLiteral("/")).last();
+
     if (importDir.exists(relativePath))
         return importDir.absoluteFilePath(relativePath);
     if (importDir.exists(withoutDir)) //Since we are in a subfolder of the import we do not require the directory
