@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,29 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "texttomodelmerger.h"
+
+#include "rewritererror.h"
+#include "modelnodepositionstorage.h"
 #include "abstractproperty.h"
 #include "bindingproperty.h"
 #include "filemanager/firstdefinitionfinder.h"
 #include "filemanager/objectlengthcalculator.h"
 #include "filemanager/qmlrefactoring.h"
-#include "filemanager/qmlwarningdialog.h"
 #include "nodeproperty.h"
 #include "propertyparser.h"
 #include "rewriterview.h"
@@ -52,10 +54,13 @@
 
 #include <QSet>
 #include <QDir>
+#include <QLoggingCategory>
 
 using namespace LanguageUtils;
 using namespace QmlJS;
 
+static Q_LOGGING_CATEGORY(rewriterBenchmark, "rewriter.load")
+
 namespace {
 
 static inline bool isSupportedAttachedProperties(const QString &propertyName)
@@ -69,7 +74,7 @@ static inline QStringList supportedVersionsList()
         QStringLiteral("2.0"), QStringLiteral("2.1"),
         QStringLiteral("2.2"), QStringLiteral("2.3"),
         QStringLiteral("2.4"), QStringLiteral("2.5"),
-        QStringLiteral("2.6")
+        QStringLiteral("2.6"), QStringLiteral("2.7")
     };
     return list;
 }
@@ -86,7 +91,8 @@ static inline QStringList globalQtEnums()
 
 static inline QStringList knownEnumScopes()
 {
-    static QStringList list = QStringList() << QStringLiteral("TextInput") << QStringLiteral("TextEdit");
+    static QStringList list = QStringList() << QStringLiteral("TextInput") << QStringLiteral("TextEdit")
+                                            << QStringLiteral("Material") << QStringLiteral("Universal") ;;
     return list;
 }
 
@@ -146,7 +152,7 @@ static inline bool isHexDigit(ushort c)
 
 static inline QString fixEscapedUnicodeChar(const QString &value) //convert "\u2939"
 {
-    if (value.count() == 6 && value.at(0) == '\\' && value.at(1) == 'u' &&
+    if (value.count() == 6 && value.at(0) == QLatin1Char('\\') && value.at(1) == QLatin1Char('u') &&
         isHexDigit(value.at(2).unicode()) && isHexDigit(value.at(3).unicode()) &&
         isHexDigit(value.at(4).unicode()) && isHexDigit(value.at(5).unicode())) {
             return convertUnicode(value.at(2).unicode(), value.at(3).unicode(), value.at(4).unicode(), value.at(5).unicode());
@@ -156,16 +162,21 @@ static inline QString fixEscapedUnicodeChar(const QString &value) //convert "\u2
 
 static inline bool isSignalPropertyName(const QString &signalName)
 {
+    if (signalName.isEmpty())
+        return false;
     // see QmlCompiler::isSignalPropertyName
-    return signalName.length() >= 3 && signalName.startsWith(QStringLiteral("on")) &&
-           signalName.at(2).isLetter();
+    QStringList list = signalName.split(QLatin1String("."));
+
+    QString pureSignalName = list.last();
+    return pureSignalName.length() >= 3 && pureSignalName.startsWith(QStringLiteral("on")) &&
+            pureSignalName.at(2).isLetter();
 }
 
 static inline QVariant cleverConvert(const QString &value)
 {
-    if (value == "true")
+    if (value == QLatin1String("true"))
         return QVariant(true);
-    if (value == "false")
+    if (value == QLatin1String("false"))
         return QVariant(false);
     bool flag;
     int i = value.toInt(&flag);
@@ -264,10 +275,14 @@ static bool isListElementType(const QmlDesigner::TypeName &type)
 
 static bool isComponentType(const QmlDesigner::TypeName &type)
 {
-    return type == "Component" || type == "Qt.Component" || type == "QtQuick.Component" || type == "<cpp>.QQmlComponent";
+    return type == "Component"
+            || type == "Qt.Component"
+            || type == "QtQuick.Component"
+            || type == "<cpp>.QQmlComponent"
+            || type == "QQmlComponent";
 }
 
-static bool isCustomParserType(const QString &type)
+static bool isCustomParserType(const QmlDesigner::TypeName &type)
 {
     return type == "QtQuick.VisualItemModel" || type == "Qt.VisualItemModel" ||
            type == "QtQuick.VisualDataModel" || type == "Qt.VisualDataModel" ||
@@ -288,7 +303,7 @@ static bool isConnectionsType(const QmlDesigner::TypeName &type)
 
 static bool propertyIsComponentType(const QmlDesigner::NodeAbstractProperty &property, const QmlDesigner::TypeName &type, QmlDesigner::Model *model)
 {
-    if (model->metaInfo(type, -1, -1).isSubclassOf("QtQuick.Component", -1, -1) && !isComponentType(type))
+    if (model->metaInfo(type).isSubclassOf("QtQuick.Component") && !isComponentType(type))
         return false; //If the type is already a subclass of Component keep it
 
     return property.parentModelNode().isValid() &&
@@ -301,7 +316,7 @@ static inline QString extractComponentFromQml(const QString &source)
         return QString();
 
     QString result;
-    if (source.contains("Component")) { //explicit component
+    if (source.contains(QLatin1String("Component"))) { //explicit component
         QmlDesigner::FirstDefinitionFinder firstDefinitionFinder(source);
         int offset = firstDefinitionFinder(0);
         if (offset < 0)
@@ -858,10 +873,17 @@ void TextToModelMerger::setupUsedImports()
 
 bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceHandler)
 {
+    qCInfo(rewriterBenchmark) << Q_FUNC_INFO;
+
+    QTime time;
+    if (rewriterBenchmark().isInfoEnabled())
+        time.start();
+
     // maybe the project environment (kit, ...) changed, so we need to clean old caches
     NodeMetaInfo::clearCache();
 
     m_qrcMapping.clear();
+    m_rewriterView->clearErrorAndWarnings();
 
     const QUrl url = m_rewriterView->model()->fileUrl();
 
@@ -880,6 +902,8 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
         doc->setSource(data);
         doc->parseQml();
 
+        qCInfo(rewriterBenchmark) << "parsed correctly: " << doc->isParsedCorrectly() << time.elapsed();
+
         if (!doc->isParsedCorrectly()) {
             QList<RewriterError> errors;
             foreach (const QmlJS::DiagnosticMessage &message, doc->diagnosticMessages())
@@ -895,6 +919,8 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
                     new ScopeChain(ctxt.scopeChain()));
         m_document = doc;
 
+        qCInfo(rewriterBenchmark) << "linked:" << time.elapsed();
+
         QList<RewriterError> errors;
         QList<RewriterError> warnings;
 
@@ -907,6 +933,7 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
 
         if (view()->checkSemanticErrors()) {
 
+
             collectSemanticErrorsAndWarnings(&errors, &warnings);
 
             if (!errors.isEmpty()) {
@@ -914,19 +941,8 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
                 setActive(false);
                 return false;
             }
-
-            /*
-             * If there are warnings and we are validating the document, then show a warning dialog.
-             * If the warning dialog is not ignored we set the warnings as errors and do not load the document
-             */
-            if (!warnings.isEmpty()
-                    && differenceHandler.isValidator()
-                    && !m_rewriterView->inErrorState()
-                    && !showWarningsDialogIgnored(warnings)) {
-                m_rewriterView->setErrors(warnings);
-                setActive(false);
-                return false;
-            }
+            m_rewriterView->setWarnings(warnings);
+            qCInfo(rewriterBenchmark) << "checked semantic errors:" << time.elapsed();
         }
         setupUsedImports();
 
@@ -937,7 +953,8 @@ bool TextToModelMerger::load(const QString &data, DifferenceHandler &differenceH
         ModelNode modelRootNode = m_rewriterView->rootModelNode();
         syncNode(modelRootNode, astRootNode, &ctxt, differenceHandler);
         m_rewriterView->positionStorage()->cleanupInvalidOffsets();
-        m_rewriterView->clearErrors();
+
+        qCInfo(rewriterBenchmark) << "synced nodes:" << time.elapsed();
 
         setActive(false);
         return true;
@@ -1091,10 +1108,16 @@ void TextToModelMerger::syncNode(ModelNode &modelNode,
                 astValue = textAt(context->doc(),
                                   property->statement->firstSourceLocation(),
                                   property->statement->lastSourceLocation());
+
+            astValue = astValue.trimmed();
+            if (astValue.endsWith(QLatin1Char(';')))
+                astValue = astValue.left(astValue.length() - 1);
+            astValue = astValue.trimmed();
+
             const TypeName &astType = property->memberType.toUtf8();
             AbstractProperty modelProperty = modelNode.property(astName.toUtf8());
             if (!property->statement || isLiteralValue(property->statement)) {
-                const QVariant variantValue = convertDynamicPropertyValueToVariant(astValue, astType);
+                const QVariant variantValue = convertDynamicPropertyValueToVariant(astValue, QString::fromLatin1(astType));
                 syncVariantProperty(modelProperty, variantValue, astType, differenceHandler);
             } else {
                 syncExpressionProperty(modelProperty, astValue, astType, differenceHandler);
@@ -1190,7 +1213,7 @@ QmlDesigner::PropertyName TextToModelMerger::syncScriptBinding(ModelNode &modelN
 {
     QString astPropertyName = toString(script->qualifiedId);
     if (!prefix.isEmpty())
-        astPropertyName.prepend(prefix + QLatin1Char('.'));
+        astPropertyName.prepend(prefix + '.');
 
     QString astValue;
     if (script->statement) {
@@ -1440,13 +1463,13 @@ ModelNode TextToModelMerger::createModelNode(const TypeName &typeName,
 
     if (isCustomParserType(typeName))
         nodeSource = textAt(context->doc(),
-                                    astObjectType->identifierToken.offset,
+                                    astObjectType->identifierToken,
                                     astNode->lastSourceLocation());
 
 
     if (isComponentType(typeName) || isImplicitComponent) {
         QString componentSource = extractComponentFromQml(textAt(context->doc(),
-                                  astObjectType->identifierToken.offset,
+                                  astObjectType->identifierToken,
                                   astNode->lastSourceLocation()));
 
 
@@ -1484,7 +1507,7 @@ QStringList TextToModelMerger::syncGroupedProperties(ModelNode &modelNode,
         AST::UiObjectMember *member = iter->member;
 
         if (AST::UiScriptBinding *script = AST::cast<AST::UiScriptBinding *>(member)) {
-            const QString prop = syncScriptBinding(modelNode, name, script, context, differenceHandler);
+            const QString prop = QString::fromLatin1(syncScriptBinding(modelNode, name, script, context, differenceHandler));
             if (!prop.isEmpty())
                 props.append(prop);
         }
@@ -1922,56 +1945,39 @@ void TextToModelMerger::collectSemanticErrorsAndWarnings(QList<RewriterError> *e
 
     check.enableQmlDesignerChecks();
 
+    QUrl fileNameUrl = QUrl::fromLocalFile(m_document->fileName());
     foreach (const StaticAnalysis::Message &message, check()) {
         if (message.severity == Severity::Error) {
             if (message.type == StaticAnalysis::ErrUnknownComponent)
-                warnings->append(RewriterError(message.toDiagnosticMessage(), QUrl::fromLocalFile(m_document->fileName())));
+                warnings->append(RewriterError(message.toDiagnosticMessage(), fileNameUrl));
             else
-                errors->append(RewriterError(message.toDiagnosticMessage(), QUrl::fromLocalFile(m_document->fileName())));
+                errors->append(RewriterError(message.toDiagnosticMessage(), fileNameUrl));
         }
         if (message.severity == Severity::Warning) {
-            if (message.type == StaticAnalysis::WarnAboutQtQuick1InsteadQtQuick2) {
-                errors->append(RewriterError(message.toDiagnosticMessage(), QUrl::fromLocalFile(m_document->fileName())));
-            } else {
-                warnings->append(RewriterError(message.toDiagnosticMessage(), QUrl::fromLocalFile(m_document->fileName())));
-            }
+            if (message.type == StaticAnalysis::WarnAboutQtQuick1InsteadQtQuick2)
+                errors->append(RewriterError(message.toDiagnosticMessage(), fileNameUrl));
+            else
+                warnings->append(RewriterError(message.toDiagnosticMessage(), fileNameUrl));
         }
     }
 }
 
-bool TextToModelMerger::showWarningsDialogIgnored(const QList<RewriterError> &warnings)
-{
-    QStringList message;
-
-    foreach (const RewriterError &warning, warnings) {
-        QString string = QStringLiteral("Line: ") +  QString::number(warning.line()) + QStringLiteral(": ")  + warning.description();
-        message << string;
-    }
-
-    QmlWarningDialog warningDialog(0, message);
-    if (warningDialog.warningsEnabled() && warningDialog.exec()) {
-        return false;
-    }
-
-    return true;
-}
-
 void TextToModelMerger::populateQrcMapping(const QString &filePath)
 {
+    if (!filePath.startsWith(QLatin1String("qrc:")))
+        return;
+
     QString path = removeFileFromQrcPath(filePath);
-    QString fileName = fileForFullQrcPath(filePath);
-    if (path.contains(QLatin1String("qrc:"))) {
-        path.remove(QLatin1String("qrc:"));
-        QMap<QString,QStringList> map = ModelManagerInterface::instance()->filesInQrcPath(path);
-        if (map.contains(fileName)) {
-            if (!map.value(fileName).isEmpty()) {
-                QString fileSystemPath =  map.value(fileName).first();
-                fileSystemPath.remove(fileName);
-                if (path.isEmpty())
-                    path.prepend(QLatin1String("/"));
-                m_qrcMapping.insert(qMakePair(path, fileSystemPath));
-            }
-        }
+    const QString fileName = fileForFullQrcPath(filePath);
+    path.remove(QLatin1String("qrc:"));
+    QMap<QString,QStringList> map = ModelManagerInterface::instance()->filesInQrcPath(path);
+    const QStringList qrcFilePathes = map.value(fileName, QStringList());
+    if (!qrcFilePathes.isEmpty()) {
+        QString fileSystemPath =  qrcFilePathes.first();
+        fileSystemPath.remove(fileName);
+        if (path.isEmpty())
+            path.prepend(QLatin1String("/"));
+        m_qrcMapping.insert(qMakePair(path, fileSystemPath));
     }
 }
 
