@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -36,6 +31,7 @@
 #include "qmljseditordocument.h"
 #include "qmljseditorplugin.h"
 #include "qmljsfindreferences.h"
+#include "qmljshighlighter.h"
 #include "qmljshoverhandler.h"
 #include "qmljsquickfixassist.h"
 #include "qmloutlinemodel.h"
@@ -57,7 +53,6 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/id.h>
-#include <coreplugin/infobar.h>
 #include <coreplugin/modemanager.h>
 
 #include <extensionsystem/pluginmanager.h>
@@ -66,22 +61,24 @@
 #include <texteditor/fontsettings.h>
 #include <texteditor/tabsettings.h>
 #include <texteditor/texteditorconstants.h>
+#include <texteditor/texteditorsettings.h>
 #include <texteditor/syntaxhighlighter.h>
 #include <texteditor/refactoroverlay.h>
 #include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/genericproposalmodel.h>
 #include <texteditor/texteditoractionhandler.h>
 
-#include <utils/changeset.h>
-#include <utils/uncommentselection.h>
-#include <utils/qtcassert.h>
 #include <utils/annotateditemdelegate.h>
+#include <utils/changeset.h>
+#include <utils/qtcassert.h>
+#include <utils/uncommentselection.h>
 
 #include <QComboBox>
 #include <QCoreApplication>
 #include <QFileInfo>
 #include <QHeaderView>
 #include <QMenu>
+#include <QMetaMethod>
 #include <QPointer>
 #include <QScopedPointer>
 #include <QSignalMapper>
@@ -111,7 +108,6 @@ QmlJSEditorWidget::QmlJSEditorWidget()
 {
     m_outlineCombo = 0;
     m_contextPane = 0;
-    m_firstSementicInfo = true;
     m_findReferences = new FindReferences(this);
 
     setLanguageSettingsId(QmlJSTools::Constants::QML_JS_SETTINGS_ID);
@@ -134,8 +130,8 @@ void QmlJSEditorWidget::finalizeInitialization()
 
     textDocument()->setCodec(QTextCodec::codecForName("UTF-8")); // qml files are defined to be utf-8
 
-    m_modelManager = QmlJS::ModelManagerInterface::instance();
-    m_contextPane = ExtensionSystem::PluginManager::getObject<QmlJS::IContextPane>();
+    m_modelManager = ModelManagerInterface::instance();
+    m_contextPane = ExtensionSystem::PluginManager::getObject<IContextPane>();
 
     m_modelManager->activateScan();
 
@@ -152,11 +148,11 @@ void QmlJSEditorWidget::finalizeInitialization()
     connect(this->document(), &QTextDocument::modificationChanged,
             this, &QmlJSEditorWidget::modificationChanged);
 
-    connect(m_qmlJsEditorDocument, SIGNAL(updateCodeWarnings(QmlJS::Document::Ptr)),
-            this, SLOT(updateCodeWarnings(QmlJS::Document::Ptr)));
+    connect(m_qmlJsEditorDocument, &QmlJSEditorDocument::updateCodeWarnings,
+            this, &QmlJSEditorWidget::updateCodeWarnings);
 
-    connect(m_qmlJsEditorDocument, SIGNAL(semanticInfoUpdated(QmlJSTools::SemanticInfo)),
-            this, SLOT(semanticInfoUpdated(QmlJSTools::SemanticInfo)));
+    connect(m_qmlJsEditorDocument, &QmlJSEditorDocument::semanticInfoUpdated,
+            this, &QmlJSEditorWidget::semanticInfoUpdated);
 
     setRequestMarkEnabled(true);
     createToolBar();
@@ -195,19 +191,20 @@ static void appendExtraSelectionsForMessages(
             sel.cursor.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, d.loc.length);
         }
 
+        const auto fontSettings = TextEditor::TextEditorSettings::instance()->fontSettings();
+
         if (d.isWarning())
-            sel.format.setUnderlineColor(Qt::darkYellow);
+            sel.format = fontSettings.toTextCharFormat(TextEditor::C_WARNING);
         else
-            sel.format.setUnderlineColor(Qt::red);
+            sel.format = fontSettings.toTextCharFormat(TextEditor::C_ERROR);
 
-        sel.format.setUnderlineStyle(QTextCharFormat::WaveUnderline);
         sel.format.setToolTip(d.message);
 
         selections->append(sel);
     }
 }
 
-void QmlJSEditorWidget::updateCodeWarnings(QmlJS::Document::Ptr doc)
+void QmlJSEditorWidget::updateCodeWarnings(Document::Ptr doc)
 {
     if (doc->ast()) {
         setExtraSelections(CodeWarningsSelection, QList<QTextEdit::ExtraSelection>());
@@ -224,7 +221,7 @@ void QmlJSEditorWidget::updateCodeWarnings(QmlJS::Document::Ptr doc)
 void QmlJSEditorWidget::modificationChanged(bool changed)
 {
     if (!changed && m_modelManager)
-        m_modelManager->fileChangedOnDisk(textDocument()->filePath());
+        m_modelManager->fileChangedOnDisk(textDocument()->filePath().toString());
 }
 
 void QmlJSEditorWidget::jumpToOutlineElement(int /*index*/)
@@ -348,7 +345,7 @@ void QmlJSEditorWidget::updateUses()
             continue;
 
         QTextEdit::ExtraSelection sel;
-        sel.format = textDocument()->fontSettings().toTextCharFormat(TextEditor::C_OCCURRENCES);
+        sel.format = textDocument()->fontSettings().toTextCharFormat(C_OCCURRENCES);
         sel.cursor = textCursor();
         sel.cursor.setPosition(loc.begin());
         sel.cursor.setPosition(loc.end(), QTextCursor::KeepAnchor);
@@ -449,7 +446,9 @@ protected:
 
 void QmlJSEditorWidget::setSelectedElements()
 {
-    if (!receivers(SIGNAL(selectedElementsChanged(QList<QmlJS::AST::UiObjectMember*>,QString))))
+    static const QMetaMethod selectedChangedSignal =
+            QMetaMethod::fromSignal(&QmlJSEditorWidget::selectedElementsChanged);
+    if (!isSignalConnected(selectedChangedSignal))
         return;
 
     QTextCursor tc = textCursor();
@@ -544,11 +543,12 @@ void QmlJSEditorWidget::createToolBar()
     policy.setHorizontalPolicy(QSizePolicy::Expanding);
     m_outlineCombo->setSizePolicy(policy);
 
-    connect(m_outlineCombo, SIGNAL(activated(int)), this, SLOT(jumpToOutlineElement(int)));
-    connect(m_qmlJsEditorDocument->outlineModel(), SIGNAL(updated()),
-            m_outlineCombo->view()/*QTreeView*/, SLOT(expandAll()));
-    connect(m_qmlJsEditorDocument->outlineModel(), SIGNAL(updated()),
-            this, SLOT(updateOutlineIndexNow()));
+    connect(m_outlineCombo, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated),
+            this, &QmlJSEditorWidget::jumpToOutlineElement);
+    connect(m_qmlJsEditorDocument->outlineModel(), &QmlOutlineModel::updated,
+            static_cast<QTreeView *>(m_outlineCombo->view()), &QTreeView::expandAll);
+    connect(m_qmlJsEditorDocument->outlineModel(), &QmlOutlineModel::updated,
+            this, &QmlJSEditorWidget::updateOutlineIndexNow);
 
     connect(this, &QmlJSEditorWidget::cursorPositionChanged,
             &m_updateOutlineIndexTimer, static_cast<void (QTimer::*)()>(&QTimer::start));
@@ -556,6 +556,166 @@ void QmlJSEditorWidget::createToolBar()
     insertExtraToolBarWidget(TextEditorWidget::Left, m_outlineCombo);
 }
 
+class CodeModelInspector : public MemberProcessor
+{
+public:
+    explicit CodeModelInspector(const CppComponentValue *processingValue, QTextStream *stream) :
+        m_processingValue(processingValue),
+        m_stream(stream),
+        m_indent(QLatin1String("    "))
+    {
+    }
+
+    bool processProperty(const QString &name, const Value *value,
+                                 const PropertyInfo &propertyInfo) override
+    {
+        QString type;
+        if (const CppComponentValue *cpp = value->asCppComponentValue())
+            type = cpp->metaObject()->className();
+        else
+            type = m_processingValue->propertyType(name);
+
+        if (propertyInfo.isList())
+            type = QStringLiteral("list<%1>").arg(type);
+
+        *m_stream << m_indent;
+        if (!propertyInfo.isWriteable())
+            *m_stream << "readonly ";
+        *m_stream << "property " << type << " " << name << endl;
+
+        return true;
+    }
+    bool processSignal(const QString &name, const Value *value) override
+    {
+        *m_stream << m_indent << "signal " << name << stringifyFunctionParameters(value) << endl;
+        return true;
+    }
+    bool processSlot(const QString &name, const Value *value) override
+    {
+        *m_stream << m_indent << "function " << name << stringifyFunctionParameters(value) << endl;
+        return true;
+    }
+    bool processGeneratedSlot(const QString &name, const Value *value) override
+    {
+        *m_stream << m_indent << "/*generated*/ function " << name
+                  << stringifyFunctionParameters(value) << endl;
+        return true;
+    }
+
+private:
+    QString stringifyFunctionParameters(const Value *value) const
+    {
+        QStringList params;
+        const QmlJS::MetaFunction *metaFunction = value->asMetaFunction();
+        if (metaFunction) {
+            QStringList paramNames = metaFunction->fakeMetaMethod().parameterNames();
+            QStringList paramTypes = metaFunction->fakeMetaMethod().parameterTypes();
+            for (int i = 0; i < paramTypes.size(); ++i) {
+                QString typeAndNamePair = paramTypes.at(i);
+                if (paramNames.size() > i) {
+                    QString paramName = paramNames.at(i);
+                    if (!paramName.isEmpty())
+                        typeAndNamePair += QLatin1Char(' ') + paramName;
+                }
+                params.append(typeAndNamePair);
+            }
+        }
+        return QLatin1Char('(') + params.join(QLatin1String(", ")) + QLatin1Char(')');
+    }
+
+private:
+    const CppComponentValue *m_processingValue;
+    QTextStream *m_stream;
+    const QString m_indent;
+};
+
+static const CppComponentValue *findCppComponentToInspect(const SemanticInfo &semanticInfo,
+                                                          const unsigned cursorPosition)
+{
+    AST::Node *node = semanticInfo.astNodeAt(cursorPosition);
+    if (!node)
+        return 0;
+
+    const ScopeChain scopeChain = semanticInfo.scopeChain(semanticInfo.rangePath(cursorPosition));
+    Evaluate evaluator(&scopeChain);
+    const Value *value = evaluator.reference(node);
+    if (!value)
+        return 0;
+
+    return value->asCppComponentValue();
+}
+
+static QString inspectCppComponent(const CppComponentValue *cppValue)
+{
+    QString result;
+    QTextStream bufWriter(&result);
+
+    // for QtObject
+    QString superClassName = cppValue->metaObject()->superclassName();
+    if (superClassName.isEmpty())
+        superClassName = cppValue->metaObject()->className();
+
+    bufWriter << "import QtQuick " << cppValue->importVersion().toString() << endl
+              << "// " << cppValue->metaObject()->className()
+              << " imported as " << cppValue->moduleName()  << " "
+              << cppValue->importVersion().toString() << endl
+              << endl
+              << superClassName << " {" << endl;
+
+    CodeModelInspector insp(cppValue, &bufWriter);
+    cppValue->processMembers(&insp);
+
+    bufWriter << endl;
+    const int enumeratorCount = cppValue->metaObject()->enumeratorCount();
+    for (int index = cppValue->metaObject()->enumeratorOffset(); index < enumeratorCount; ++index) {
+        LanguageUtils::FakeMetaEnum enumerator = cppValue->metaObject()->enumerator(index);
+        bufWriter << "    // Enum " << enumerator.name() << " { " <<
+                     enumerator.keys().join(QLatin1Char(',')) << " }" << endl;
+    }
+
+    bufWriter << "}" << endl;
+    return result;
+}
+
+void QmlJSEditorWidget::inspectElementUnderCursor() const
+{
+    const QTextCursor cursor = textCursor();
+
+    const unsigned cursorPosition = cursor.position();
+    const SemanticInfo semanticInfo = m_qmlJsEditorDocument->semanticInfo();
+    if (!semanticInfo.isValid())
+        return;
+
+    const CppComponentValue *cppValue = findCppComponentToInspect(semanticInfo, cursorPosition);
+    if (!cppValue) {
+        QString title = tr("Code Model Not Available");
+        const QString nothingToShow = QStringLiteral("nothingToShow");
+        EditorManager::openEditorWithContents(Core::Constants::K_DEFAULT_TEXT_EDITOR_ID, &title,
+                                              tr("Code model not available.").toUtf8(), nothingToShow,
+                                              EditorManager::IgnoreNavigationHistory);
+        return;
+    }
+
+    QString title = tr("Code Model of %1").arg(cppValue->metaObject()->className());
+    IEditor *outputEditor = EditorManager::openEditorWithContents(
+                Core::Constants::K_DEFAULT_TEXT_EDITOR_ID, &title, QByteArray(),
+                cppValue->metaObject()->className(), EditorManager::IgnoreNavigationHistory);
+
+    if (!outputEditor)
+        return;
+
+    auto widget = qobject_cast<TextEditor::TextEditorWidget *>(outputEditor->widget());
+    if (!widget)
+        return;
+
+    widget->setReadOnly(true);
+    widget->textDocument()->setTemporary(true);
+    widget->textDocument()->setSyntaxHighlighter(new QmlJSHighlighter(widget->document()));
+
+    const QString buf = inspectCppComponent(cppValue);
+    widget->textDocument()->setPlainText(buf);
+}
+
 TextEditorWidget::Link QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
                                                              bool /*resolveTarget*/,
                                                              bool /*inNextSplit*/)
@@ -641,12 +801,12 @@ TextEditorWidget::Link QmlJSEditorWidget::findLinkAt(const QTextCursor &cursor,
 
 void QmlJSEditorWidget::findUsages()
 {
-    m_findReferences->findUsages(textDocument()->filePath(), textCursor().position());
+    m_findReferences->findUsages(textDocument()->filePath().toString(), textCursor().position());
 }
 
 void QmlJSEditorWidget::renameUsages()
 {
-    m_findReferences->renameUsages(textDocument()->filePath(), textCursor().position());
+    m_findReferences->renameUsages(textDocument()->filePath().toString(), textCursor().position());
 }
 
 void QmlJSEditorWidget::showContextPane()
@@ -663,19 +823,12 @@ void QmlJSEditorWidget::showContextPane()
     }
 }
 
-void QmlJSEditorWidget::performQuickFix(int index)
-{
-    m_quickFixes.at(index)->perform();
-}
-
 void QmlJSEditorWidget::contextMenuEvent(QContextMenuEvent *e)
 {
     QPointer<QMenu> menu(new QMenu(this));
 
     QMenu *refactoringMenu = new QMenu(tr("Refactoring"), menu);
 
-    QSignalMapper mapper;
-    connect(&mapper, SIGNAL(mapped(int)), this, SLOT(performQuickFix(int)));
     if (!m_qmlJsEditorDocument->isSemanticInfoOutdated()) {
         AssistInterface *interface = createAssistInterface(QuickFix, ExplicitlyInvoked);
         if (interface) {
@@ -687,10 +840,8 @@ void QmlJSEditorWidget::contextMenuEvent(QContextMenuEvent *e)
                 for (int index = 0; index < model->size(); ++index) {
                     AssistProposalItem *item = static_cast<AssistProposalItem *>(model->proposalItem(index));
                     QuickFixOperation::Ptr op = item->data().value<QuickFixOperation::Ptr>();
-                    m_quickFixes.append(op);
                     QAction *action = refactoringMenu->addAction(op->description());
-                    mapper.setMapping(action, index);
-                    connect(action, SIGNAL(triggered()), &mapper, SLOT(map()));
+                    connect(action, &QAction::triggered, this, [op]() { op->perform(); });
                 }
                 delete model;
             }
@@ -717,9 +868,6 @@ void QmlJSEditorWidget::contextMenuEvent(QContextMenuEvent *e)
     appendStandardContextMenuActions(menu);
 
     menu->exec(e->globalPos());
-    if (!menu)
-        return;
-    m_quickFixes.clear();
     delete menu;
 }
 
@@ -788,16 +936,6 @@ void QmlJSEditorWidget::semanticInfoUpdated(const SemanticInfo &semanticInfo)
         }
     }
 
-    if (m_firstSementicInfo) {
-        m_firstSementicInfo = false;
-        if (semanticInfo.document->language() == Dialect::QmlQtQuick2Ui) {
-            Core::InfoBarEntry info(Core::Id(Constants::QML_UI_FILE_WARNING),
-                                    tr("This file should only be edited in <b>Design</b> mode."));
-            info.setCustomButtonInfo(tr("Switch Mode"), []() { ModeManager::activateMode(Core::Constants::MODE_DESIGN); });
-            textDocument()->infoBar()->addInfo(info);
-        }
-    }
-
     updateUses();
 }
 
@@ -840,16 +978,16 @@ bool QmlJSEditorWidget::hideContextPane()
 }
 
 AssistInterface *QmlJSEditorWidget::createAssistInterface(
-    TextEditor::AssistKind assistKind,
-    TextEditor::AssistReason reason) const
+    AssistKind assistKind,
+    AssistReason reason) const
 {
-    if (assistKind == TextEditor::Completion) {
+    if (assistKind == Completion) {
         return new QmlJSCompletionAssistInterface(document(),
                                                   position(),
-                                                  textDocument()->filePath(),
+                                                  textDocument()->filePath().toString(),
                                                   reason,
                                                   m_qmlJsEditorDocument->semanticInfo());
-    } else if (assistKind == TextEditor::QuickFix) {
+    } else if (assistKind == QuickFix) {
         return new QmlJSQuickFixAssistInterface(const_cast<QmlJSEditorWidget *>(this), reason);
     }
     return 0;
@@ -868,7 +1006,7 @@ QString QmlJSEditorWidget::foldReplacementText(const QTextBlock &block) const
             return QLatin1String("id: ") + objectId + QLatin1String("...");
     }
 
-    return TextEditor::TextEditorWidget::foldReplacementText(block);
+    return TextEditorWidget::foldReplacementText(block);
 }
 
 
@@ -890,9 +1028,8 @@ bool QmlJSEditor::isDesignModePreferred() const
         alwaysPreferDesignMode = true;
 
     // stay in design mode if we are there
-    IMode *mode = ModeManager::currentMode();
-    return alwaysPreferDesignMode
-            || (mode && mode->id() == Core::Constants::MODE_DESIGN);
+    Id mode = ModeManager::currentMode();
+    return alwaysPreferDesignMode || mode == Core::Constants::MODE_DESIGN;
 }
 
 
