@@ -68,6 +68,7 @@ void FontSettings::clear()
     m_antialias = DEFAULT_ANTIALIAS;
     m_scheme.clear();
     m_formatCache.clear();
+    m_textCharFormatCache.clear();
 }
 
 void FontSettings::toSettings(const QString &category,
@@ -180,12 +181,9 @@ QTextCharFormat FontSettings::toTextCharFormat(TextStyle category) const
     return tf;
 }
 
-uint qHash(const TextStyles &textStyles)
+uint qHash(TextStyles textStyles)
 {
-    uint hash = qHash(textStyles.mainStyle);
-    for (TextStyle mixinStyle : textStyles.mixinStyles)
-        hash ^= qHash(mixinStyle);
-    return hash;
+    return ::qHash(reinterpret_cast<quint64&>(textStyles));
 }
 
 bool operator==(const TextStyles &first, const TextStyles &second)
@@ -218,7 +216,7 @@ void FontSettings::addMixinStyle(QTextCharFormat &textCharFormat,
     };
 }
 
-QTextCharFormat FontSettings::toTextCharFormat(const TextStyles textStyles) const
+QTextCharFormat FontSettings::toTextCharFormat(TextStyles textStyles) const
 {
     auto textCharFormatIterator = m_textCharFormatCache.find(textStyles);
     if (textCharFormatIterator != m_textCharFormatCache.end())
@@ -259,6 +257,7 @@ void FontSettings::setFamily(const QString &family)
 {
     m_family = family;
     m_formatCache.clear();
+    m_textCharFormatCache.clear();
 }
 
 /**
@@ -273,6 +272,7 @@ void FontSettings::setFontSize(int size)
 {
     m_fontSize = size;
     m_formatCache.clear();
+    m_textCharFormatCache.clear();
 }
 
 /**
@@ -287,6 +287,7 @@ void FontSettings::setFontZoom(int zoom)
 {
     m_fontZoom = zoom;
     m_formatCache.clear();
+    m_textCharFormatCache.clear();
 }
 
 QFont FontSettings::font() const
@@ -308,6 +309,7 @@ void FontSettings::setAntialias(bool antialias)
 {
     m_antialias = antialias;
     m_formatCache.clear();
+    m_textCharFormatCache.clear();
 }
 
 /**
@@ -345,6 +347,7 @@ bool FontSettings::loadColorScheme(const QString &fileName,
                                    const FormatDescriptions &descriptions)
 {
     m_formatCache.clear();
+    m_textCharFormatCache.clear();
     bool loaded = true;
     m_schemeFileName = fileName;
 
@@ -359,12 +362,20 @@ bool FontSettings::loadColorScheme(const QString &fileName,
         const TextStyle id = desc.id();
         if (!m_scheme.contains(id)) {
             Format format;
-            format.setForeground(desc.foreground());
-            format.setBackground(desc.background());
-            format.setBold(desc.format().bold());
-            format.setItalic(desc.format().italic());
-            format.setUnderlineColor(desc.format().underlineColor());
-            format.setUnderlineStyle(desc.format().underlineStyle());
+            const Format &descFormat = desc.format();
+            if (descFormat == format && m_scheme.contains(C_TEXT)) {
+                // Default format -> Text
+                const Format textFormat = m_scheme.formatFor(C_TEXT);
+                format.setForeground(textFormat.foreground());
+                format.setBackground(textFormat.background());
+            } else {
+                format.setForeground(descFormat.foreground());
+                format.setBackground(descFormat.background());
+            }
+            format.setBold(descFormat.bold());
+            format.setItalic(descFormat.italic());
+            format.setUnderlineColor(descFormat.underlineColor());
+            format.setUnderlineStyle(descFormat.underlineStyle());
             m_scheme.setFormatFor(id, format);
         }
     }
@@ -392,6 +403,7 @@ void FontSettings::setColorScheme(const ColorScheme &scheme)
 {
     m_scheme = scheme;
     m_formatCache.clear();
+    m_textCharFormatCache.clear();
 }
 
 static QString defaultFontFamily()
