@@ -3,7 +3,7 @@
 ** Copyright (C) 2015 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing
 **
-** This file is part of the Qt Build Suite.
+** This file is part of Qbs.
 **
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
@@ -40,10 +40,13 @@
 namespace qbs {
 namespace Internal {
 
+class ProgressObserver;
+
 class BuildGraphLocker
 {
 public:
-    explicit BuildGraphLocker(const QString &buildGraphFilePath, const Logger &logger);
+    explicit BuildGraphLocker(const QString &buildGraphFilePath, const Logger &logger,
+                              bool waitIndefinitely, ProgressObserver *observer);
     ~BuildGraphLocker();
 
 private:
